/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.common.utils;

import it.fe.core.par.invoice.AdditionalData;
import it.fe.core.par.invoice.Invoice;
import it.fe.core.par.invoice.InvoiceLine;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.SupplyInfo;

import java.util.ArrayList;
import java.util.List;

public class InvoiceUtils {
	
	public List<MeterSummary> filterMeterSummariesByPreviousDocument(List<MeterSummary> meterSummaries, PreviousDocument document) {
		List<MeterSummary> result = new ArrayList<MeterSummary>();
		
		for (MeterSummary meterSummary : meterSummaries) {
			for (SupplyInfo supplyInfo : meterSummary.getMeter().getSupplyInfo()) {
				// TODO: per ora basta il match anche su un solo supplyInfo
				if (supplyInfo.getPreviousDocument().getPreviousDocumentId() == document.getPreviousDocumentId()) {
					result.add(meterSummary);
					break;
				}
			}
		}
		
		return result;
		
	}
	
	/**
	 * Method used to retrieve and return additional data by class type
	 * 
	 * @param invoice
	 * @param clazz
	 * @return
	 */
	public static boolean invoiceHasSavedAdditionalData(Invoice invoice, Class clazz) {
		boolean result = false;
		
		List<AdditionalData> additionalData = invoice.getAdditionalData();
		
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(invoice);
		} 
		if ( (additionalData!=null) && (!additionalData.isEmpty())) {
			for (AdditionalData ad : additionalData) {
				if (ad.getClass().equals(clazz)) {
					result = true;
					break;
				}
			}
		}
		return result;
	}

	/**
	 * @param invoiceLine
	 * @param clazz
	 * @return
	 */
	public static boolean invoiceLineHasSavedAdditionalData(InvoiceLine invoiceLine, Class clazz) {
		boolean result = false;
		List<AdditionalData> additionalData = invoiceLine.getAdditionalData();
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceLineAdditionalData(invoiceLine);
		} 
		if ( (additionalData!=null) && (!additionalData.isEmpty()) ) {
			for (AdditionalData ad : additionalData) {
				if (ad.getClass().equals(clazz)) {
					result = true;
					break;
				}
			}
		}
		return result;
	}
	
	/**
	 * Helper method used to replace an AdditionalData within AdditionalData invoice's list.
	 * 
	 * 
	 * @param invoice
	 * @param newAdditionalData
	 * @param clazz (this param is needed since if newAdditionalData is null it's impossible to determine the AdditionalData type to be removed)
	 * @return
	 */
	public static Invoice replaceAdditionalData(Invoice invoice, AdditionalData newAdditionalData, Class clazz) {
		List<AdditionalData> additionalDataList = invoice.getAdditionalData();
		if ((additionalDataList != null) && (!additionalDataList.isEmpty())) {
			
			for (AdditionalData ad : additionalDataList) {
				if(ad.getClass().equals(clazz.getClass())) {
					if (newAdditionalData == null)
						additionalDataList.remove(additionalDataList.indexOf(ad));
					else 
						additionalDataList.set(additionalDataList.indexOf(ad), newAdditionalData);
				}
			}
			
		}
		invoice.setAdditionalData(additionalDataList);
		return invoice;
	}
	
	/**
	 * Helper method used to add, save or replace an AdditionalData within AdditionalData invoice's list.
	 * 
	 * @param invoice
	 * @param newAdditionalData
	 * @param clazz
	 * @return
	 */
	public static Invoice saveOrReplaceAdditionalData(Invoice invoice, AdditionalData newAdditionalData, Class clazz) {
		List<AdditionalData> additionalDataList = invoice.getAdditionalData();
		if ((additionalDataList != null)) {
			
			// il dato non è nullo e non è presente fra i dati aggiuntivi -> lo aggiungo
			if( (newAdditionalData != null) && (!additionalDataList.contains(newAdditionalData))) {
				additionalDataList.add(newAdditionalData);
			}
			
			// il dato può essere nullo o già presente fra i dati aggiuntivi
			else {
				synchronized (additionalDataList) {
					for (AdditionalData ad : additionalDataList) {
						if(ad.getClass().equals(clazz)) {
							if (newAdditionalData == null)
								additionalDataList.remove(additionalDataList.indexOf(ad));
							else 
								additionalDataList.set(additionalDataList.indexOf(ad), newAdditionalData);
						}
					}
				}
			}
			
		}
		invoice.setAdditionalData(additionalDataList);
		return invoice;
	}
	
	public static InvoiceLine saveOrReplaceAdditionalData(InvoiceLine invoiceLine, AdditionalData newAdditionalData, Class clazz) {
		List<AdditionalData> additionalDataList = invoiceLine.getAdditionalData();
		if ((additionalDataList != null)) {
			
			// il dato non è nullo e non è presente fra i dati aggiuntivi -> lo aggiungo
			if( (newAdditionalData != null) && (!additionalDataList.contains(newAdditionalData))) {
				additionalDataList.add(newAdditionalData);
			}
			
			// il dato può essere nullo o già presente fra i dati aggiuntivi
			else {
				synchronized (additionalDataList) {
					for (AdditionalData ad : additionalDataList) {
						if(ad.getClass().equals(clazz)) {
							if (newAdditionalData == null)
								additionalDataList.remove(additionalDataList.indexOf(ad));
							else 
								additionalDataList.set(additionalDataList.indexOf(ad), newAdditionalData);
						}
					}
				}
			}
			
		}
		invoiceLine.setAdditionalData(additionalDataList);
		return invoiceLine;
	}

	/**
	 * Method used to retrieve and return additional data by class type
	 * 
	 * @param <T>
	 * @param invoiceLine
	 * @param clazz
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static <T> List<T> getInvoiceLineStoredAdditionalData(InvoiceLine invoiceLine, Class<T> clazz) {
		
		List<T> result = new ArrayList<T>();

		List<AdditionalData> additionalData = invoiceLine.getAdditionalData();
		
		if (additionalData != null && !additionalData.isEmpty()) {
			for (AdditionalData ad : additionalData) {
				if (ad.getClass().equals(clazz)) {
					result.add((T) ad);
				}
			}
		}

		return result;
		
	}

	@SuppressWarnings("unchecked")
	public static <T> List<T> getInvoiceStoredAdditionalData(Invoice invoice,
			Class<T> clazz) {
		List<T> result = new ArrayList<T>();

		List<AdditionalData> additionalData = invoice.getAdditionalData();
		
		// fetching lazy data (additionalData may be a Hibernate Persistent Bag proxy. in this case i have to fetch data)
		if (! (additionalData instanceof ArrayList) ) {
			additionalData = JNDIUtils.getInvoiceManager().fetchInvoiceAdditionalData(invoice);
		} 

		for (AdditionalData ad : additionalData) {

			if (ad.getClass().equals(clazz)) {
				result.add((T) ad);
			}

		}
		return result;
	}

}
