/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */

package it.fe.core.common.utils;

import it.fe.core.par.invoice.AllowanceCharge;
import it.fe.core.par.invoice.PreviousDocument;
import it.fe.core.par.invoice.TaxDetail;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;


/**
 * @author chiara@epocaricerca.it
 * 
 * Costruzione di una fattura XMl in formato UBL-like
 *
 */
public class XmlFactory {
	
	public enum itemKeyword { CONSUMO, LETTURA, RIASSUNTO_CONTATORE, RIGA }
	
	//Namespace Invoice
	static Namespace xsd = Namespace.getNamespace("xsd", "http://www.cdisc.org/ns/odm/v1.2");
	static Namespace xsi = Namespace.getNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
	static Namespace qdt = Namespace.getNamespace("qdt", "urn:oasis:names:specification:ubl:schema:xsd:QualifiedDatatypes-2");
	static Namespace ccts = Namespace.getNamespace("ccts", "urn:oasis:names:specification:ubl:schema:xsd:CoreComponentParameters-2");
	static Namespace stat = Namespace.getNamespace("stat", "urn:oasis:names:specification:ubl:schema:xsd:DocumentStatusCode-1.0");
	static Namespace cbc = Namespace.getNamespace("cbc", "urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2");
	static Namespace cac = Namespace.getNamespace("cac", "urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2");
	static Namespace udt = Namespace.getNamespace("udt", "urn:un:unece:uncefact:data:draft:UnqualifiedDataTypesSchemaModule:2");
	static Namespace ext = Namespace.getNamespace("ext","urn:oasis:names:specification:ubl:schema:xsd:CommonExtensionComponents-2");
	static Namespace invoice = Namespace.getNamespace("urn:oasis:names:specification:ubl:schema:xsd:Invoice-2");	
	
	//Namespace Utility Statement
	static Namespace usCcts = Namespace.getNamespace("ccts", "urn:un:unece:uncefact:documentation:2");
	static Namespace utilityStatementNamespace = Namespace.getNamespace("urn:oasis:names:specification:ubl:schema:xsd:UtilityStatement-2");
	
	public XmlFactory() {

	}
	
	/**
	 * Crea il documento xml al quale verranno aggiunti i tag relativi alla fattura.
	 * <br />
	 * Vengono settati la radice (tag <i>Invoice</i>) ed i namespace.
	 * 
	 * @param (void)
	 * @return (Element)
	 */
	public static Element createDocument(){
		//Elemento radice
		Element root = new Element("Invoice");
		
		root.addNamespaceDeclaration(xsd);
		root.addNamespaceDeclaration(xsi);
		root.setAttribute("schemaLocation", "urn:oasis:names:specification:ubl:schema:xsd:Invoice-2 http://docs.oasis-open.org/ubl/os-UBL-2.0/xsd/maindoc/UBL-Invoice-2.0.xsd", xsi);
		root.addNamespaceDeclaration(qdt);
		root.addNamespaceDeclaration(ccts);
		root.addNamespaceDeclaration(stat);
		root.addNamespaceDeclaration(cbc);
		root.addNamespaceDeclaration(cac);
		root.addNamespaceDeclaration(udt);
		root.addNamespaceDeclaration(ext);
		root.setNamespace(invoice);
		
		return root;
	}
	
	
	/**
	 * Genera l'output dell'xml
	 * 
	 * @param root - (Element) elemento radice, nel quale sono annidati tutti gli elementi costitutivi della fattura
	 * @return (byte[]) byte array del file xml generato
	 * 
	 */
	public static byte[] generateXml(Element root) { 
		Document document = new Document(root);
		byte[] out;
		
		//crea un documento XML indentato
		XMLOutputter outputter = new XMLOutputter(Format.getPrettyFormat());
		
		String xmlOk = SpecialCharacter.adjustXmlEntity(outputter.outputString(document));
		out = xmlOk.getBytes();
		
		return out;
	}
	
	/**
	 * Aggiunge il tag <i>cbc:UBLVersionID</i>
	 * 
	 * @param ublVersionId (String) versione dell'UBL <br />
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setUBLVersionID(Element root, String ublVersionId) {
		//UBL VersionID
		//TODO da gestire con file properties
		Element cbcUBLVersionID = new Element("UBLVersionID").setNamespace(cbc);
		root.addContent(cbcUBLVersionID);
		cbcUBLVersionID.setText(ublVersionId);
	}
	
	/**
	 * Aggiunge il tag <i>cbc:CustomizationID</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setCustomizationID(Element root) {
		Element cbcCustomizationID = new Element("CustomizationID").setNamespace(cbc);
		root.addContent(cbcCustomizationID);
		cbcCustomizationID.setText("urn:www.cenbii.eu:transaction:biicoretrdm001:ver1.0");
	}
	
	/**
	 * Aggiunge il tag <i>cbc:ProfileID</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setProfileID(Element root) {
		Element cbcProfileID = new Element("ProfileID").setNamespace(cbc);
		root.addContent(cbcProfileID);
		cbcProfileID.setText("urn:www.cenbii.eu:profile:bii04:ver1.0");
	}
	
	public static void setInvoiceTypeCode(Element root, String invoiceTypeCode) {
		//InvoiceType
		//rif. http://www.esv.se/download/18.6dae77a0113497f158680002577/NES+Code+Lists+and+Identification+Schemes+-+Version+2.pdf
		//TODO da gestire con file properties
		Element cbcInvoiceTypeCode = new Element("InvoiceTypeCode").setNamespace(cbc);
		root.addContent(cbcInvoiceTypeCode);
		cbcInvoiceTypeCode.setText(invoiceTypeCode); //380: Commercial Invoice
	}

	/**
	 * Aggiunge il tag <i>cbc:Id</i>
	 * 
	 * @param numeroFattura (String) numero della fattura (lato fornitore) <br />
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setNumeroFattura(Element root, String numeroFattura) {
		Element cbcId = new Element("ID").setNamespace(cbc);
		root.addContent(cbcId);
		cbcId.setText(numeroFattura);
	}
	

	/**
	 * Aggiunge il tag <i>cbc:IssueDate</i>
	 * 
	 * @param dataEmissione (String) data di emissione della fattura <br />
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setDataEmissione(Element root, String dataEmissione) {
		Element cbcIssueDate = new Element("IssueDate").setNamespace(cbc);
		root.addContent(cbcIssueDate);
		cbcIssueDate.setText(formatDateString(dataEmissione));
	}
	
	/**
	 * Aggiunge il tag <i>cbc:Note</i>
	 * 
	 * @param note (String) note della fattura <br />
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setInvoiceNote(Element root, String note) {
		Element cbcNote = new Element("Note").setNamespace(cbc);
		root.addContent(cbcNote);
		cbcNote.setText(note);
	}
	
	/**
	 * Aggiunge il tag <i>cbc:ContoContrattuale</i>
	 * 
	 * @param contoContrattuale (String) il conto contrattuale <br />
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setContoContrattuale(Element root, String contoContrattuale) {
		Element cbcContoContrattuale = new Element("ContoContrattuale").setNamespace(cbc);
		root.addContent(cbcContoContrattuale);
		cbcContoContrattuale.setText(contoContrattuale);
	}
	
	/**
	 * Aggiunge il tag <i>cbc:DocumentCurrencyCode</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setDocumentCurrencyCode(Element root) {
		Element cbcDocumentCurrencyCode = new Element("DocumentCurrencyCode").setNamespace(cbc);
		root.addContent(cbcDocumentCurrencyCode);
		cbcDocumentCurrencyCode.setText("EUR");
	}
	
	/**
	 * Aggiunge il tag <i>cbc:TaxPointDate</i>
	 * 
	 * @param date (String) data inizio tasse <br />
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setTaxPointDate(Element root, String date) {
		Element cbcTaxPointDate = new Element("TaxPointDate").setNamespace(cbc);
		root.addContent(cbcTaxPointDate);
		cbcTaxPointDate.setText(formatDateString(date));
	}
	
	/**
	 * Aggiunge il tag <i>cbc:AccountingCost</i>
	 * 
	 * @param accountingCost (String) centro di costo <br />
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setAccountingCost(Element root, String accountingCost) {
		Element cbcAccountingCost = new Element("AccountingCost").setNamespace(cbc);
		root.addContent(cbcAccountingCost);
		cbcAccountingCost.setText(accountingCost);
	}
	
	/**
	 * Aggiunge il tag <i>cac:InvoicePeriod</i>
	 * 
	 * @param dataInizio (String) data inizio periodo di consumo <br />
	 * @param dataFine (String) data fine periodo di consumo <br />
	 * @param codicePeriodo (String) codice periodo di consumo (es. bimestre 02) <br />
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setPeriodoFatturazione(Element root, String dataInizio, String dataFine, String codicePeriodo) {
		Element cacInvoicePeriod = new Element("InvoicePeriod").setNamespace(cac);
		Element cbcStartDate = new Element("StartDate").setNamespace(cbc);
		Element cbcEndDate = new Element("EndDate").setNamespace(cbc);
		Element cbcDescriptionCode = new Element("DescriptionCode").setNamespace(cbc);
		Element cbcDescription = new Element("Description").setNamespace(cbc);
		
		//TODO conversione date e calcolo periodo se stringa vuota
		
		root.addContent(cacInvoicePeriod);
		//Data inizio periodo
		if((dataInizio != null) && (!dataInizio.equals(""))){
			cacInvoicePeriod.addContent(cbcStartDate);
			cbcStartDate.setText(formatDateString(dataInizio));
		}
		//Data fine periodo
		if((dataFine != null) && (!dataFine.equals(""))){
			cacInvoicePeriod.addContent(cbcEndDate);
			cbcEndDate.setText(formatDateString(dataFine));
		}
		//Codide periodo
		if((codicePeriodo != null) && (!codicePeriodo.equals(""))){
			cacInvoicePeriod.addContent(cbcDescriptionCode);
			cbcDescriptionCode.setText(codicePeriodo);
		}
		//Descrizione
//		cacInvoicePeriod.addContent(cbcDescription);
//		cbcDescription.setText("Periodo di consumo");
	}
	
	/**
	 * Aggiunge il tag <i>cac:OrderReference</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param orderCode (String) numero del contratto riportato in fattura
	 * 
	 * @return (void)
	 */
	public static void setOrderReference(Element root, String orderCode) {
		Element cacOrderReference = new Element("OrderReference").setNamespace(cac);
		Element cbcID = new Element("ID").setNamespace(cbc);
		
		root.addContent(cacOrderReference);
		//ID dell'ordine
		cacOrderReference.addContent(cbcID);
		cbcID.setText(orderCode);
	}
	
	/**
	 * Aggiunge il tag <i>cac:ContractDocumentReference</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param numContratto (String) numero del contratto riportato in fattura
	 * @param documentType (String) causale del contratto (es. settore merceologico: acqua)
	 * 
	 * @return (void)
	 */
	public static void setContrattoDiRiferimento(Element root, String numContratto, String documentType) {
		Element cacContractDocumentReference = new Element("ContractDocumentReference").setNamespace(cac);
		Element cbcID = new Element("ID").setNamespace(cbc);
		
		Element cbcDocumentType = new Element("DocumentType").setNamespace(cbc);

		root.addContent(cacContractDocumentReference);
		//ID del contratto
		cacContractDocumentReference.addContent(cbcID);
		cbcID.setText(numContratto);
		
		cacContractDocumentReference.addContent(cbcDocumentType);
		cbcDocumentType.setText(documentType);
	}
	
	/**
	 * Aggiunge il tag <i>cac:AdditionalDocumentReference</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param documentCode (String) numero del documento riportato in fattura
	 * @param documentType (String) causale del contratto (es. settore merceologico: acqua)
	 * @param documentURI (String) URI del documento
	 * 
	 * @return (void)
	 */
	public static void setAdditionalDocumentReference(Element root, String documentCode, String documentType, String documentURI) {
		Element cacAdditionalDocumentReference = new Element("AdditionalDocumentReference").setNamespace(cac);
		Element cbcID = new Element("ID").setNamespace(cbc);
		
		Element cbcDocumentType = new Element("DocumentType").setNamespace(cbc);
		
		Element cacAttachment = new Element("Attachment").setNamespace(cac);
		Element cacExternalReference = new Element("ExternalReference").setNamespace(cac);
		Element cbcURI = new Element("URI").setNamespace(cbc);

		root.addContent(cacAdditionalDocumentReference);
		//ID del contratto
		cacAdditionalDocumentReference.addContent(cbcID);
		cbcID.setText(documentCode);
		
		if(documentType != null){
			cacAdditionalDocumentReference.addContent(cbcDocumentType);
			cbcDocumentType.setText(documentType);
		}
		
		if(documentURI != null){
			cacAdditionalDocumentReference.addContent(cacAttachment);
			cacAttachment.addContent(cacExternalReference);
			cacExternalReference.addContent(cbcURI);
			cbcURI.setText(documentURI);
		}
		
		//TODO embeddedDocument
	}
	
	
	/**
	 * Aggiunge il tag <i>cbc:AccountingSupplierParty</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param codiceFornitore 
	 * @param ragioneSociale 
	 * @param indirizzo 
	 * @param civico 
	 * @param citta 
	 * @param cap 
	 * @param codiceNazione 
	 * @param nazione 
	 * @param nomeRegistrazione 
	 * @param partitaIva 
	 * 
	 * @return (void)
	 */
	public static void setFornitore(Element root, String codiceFornitore, String ragioneSociale, String endpointId, 
										String indirizzo, String civico, String citta, String cap, String provincia, String codiceNazione, String nazione, 
										String nomeRegistrazione, String indirizzoSedeLegale, String civicoSedeLegale, String cittaSedeLegale, String capSedeLegale, 
										String provinciaSedeLegale, String codiceNazioneSedeLegale, String nazioneSedeLegale, String partitaIva) {
		
		Element cacAccountingSupplierParty = new Element("AccountingSupplierParty").setNamespace(cac);
		Element cbcCustomerAssignedAccountID = new Element("CustomerAssignedAccountID").setNamespace(cbc);
		Element cbcEndpointID = new Element("EndpointID").setNamespace(cbc);

		root.addContent(cacAccountingSupplierParty);
		
		if((codiceFornitore != null) && (!codiceFornitore.equals(""))){
			cacAccountingSupplierParty.addContent(cbcCustomerAssignedAccountID);
			cbcCustomerAssignedAccountID.setText(codiceFornitore);
		}
		
		if((endpointId != null) && (!endpointId.equals(""))){
			cacAccountingSupplierParty.addContent(cbcEndpointID);
			cbcEndpointID.setText(endpointId);
		}
		
		setDatiParte(cacAccountingSupplierParty, ragioneSociale,
				indirizzo, civico, citta, cap, provincia, codiceNazione, nazione, 
				nomeRegistrazione, indirizzoSedeLegale, civicoSedeLegale, cittaSedeLegale, capSedeLegale, provinciaSedeLegale, codiceNazioneSedeLegale, nazioneSedeLegale, partitaIva);
	}
	
	/**
	 * Aggiunge il tag <i>cbc:AccountingCustomerParty</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param codiceFornitore 
	 * @param ragioneSociale 
	 * @param indirizzo 
	 * @param civico 
	 * @param citta 
	 * @param cap 
	 * @param codiceNazione 
	 * @param nazione 
	 * @param registrazione 
	 * @param partitaIva 
	 * 
	 * @return (void)
	 */
	public static void setCliente(Element root, String codiceCliente, String ragioneSociale, 
			String indirizzo, String civico, String citta, String cap, String provincia, String codiceNazione, String nazione, 
			String nomeRegistrazione, String indirizzoSedeLegale, String civicoSedeLegale, String cittaSedeLegale, String capSedeLegale, 
			String provinciaSedeLegale, String codiceNazioneSedeLegale, String nazioneSedeLegale, String partitaIva) {
		
		Element cacAccountingCustomerParty = new Element("AccountingCustomerParty").setNamespace(cac);
		Element cbcSupplierAssignedAccountID = new Element("SupplierAssignedAccountID").setNamespace(cbc);


		root.addContent(cacAccountingCustomerParty);
		
		cacAccountingCustomerParty.addContent(cbcSupplierAssignedAccountID);
		cbcSupplierAssignedAccountID.setText(codiceCliente);
		
		setDatiParte(cacAccountingCustomerParty, ragioneSociale,
				indirizzo, civico, citta, cap, provincia, codiceNazione, nazione, 
				nomeRegistrazione, indirizzoSedeLegale, civicoSedeLegale, cittaSedeLegale, capSedeLegale, provinciaSedeLegale, codiceNazioneSedeLegale, nazioneSedeLegale, partitaIva);
	}

	private static void setDatiParte(Element cbcAccounting_Party, String ragioneSociale, 
										String indirizzo, String civico, String citta, String cap, String provincia, String codiceNazione, String nazione, 
										String registrazione, String indirizzoSedeLegale, String civicoSedeLegale, String cittaSedeLegale, String capSedeLegale, 
										String provinciaSedeLegale, String codiceNazioneSedeLegale, String nazioneSedeLegale, String partitaIva){
		Element cacParty = new Element("Party").setNamespace(cac);
		Element cacPartyName = new Element("PartyName").setNamespace(cac);
		Element cbcName = new Element("Name").setNamespace(cbc);		
		
		Element cacPostalAddress = new Element("PostalAddress").setNamespace(cac);
		
		cbcAccounting_Party.addContent(cacParty);
		cacParty.addContent(cacPartyName);
		cacPartyName.addContent(cbcName);
		cbcName.setText(ragioneSociale);
		
		setIndirizzo(cacParty, cacPostalAddress, indirizzo, civico, citta, cap, provincia, codiceNazione, nazione);

		Element cacPartyTaxScheme = new Element("PartyTaxScheme").setNamespace(cac);
		Element cbcCompanyId = new Element("CompanyID").setNamespace(cbc);
		
		cacParty.addContent(cacPartyTaxScheme);
		cbcCompanyId.setAttribute(new Attribute("schemeID", "IT:VAT"));
		cbcCompanyId.setText(partitaIva);
		cacPartyTaxScheme.addContent(cbcCompanyId);
		setTaxScheme(cacPartyTaxScheme);
		
		setDatiLegaliParte(cacParty, registrazione, 
				indirizzoSedeLegale, civicoSedeLegale, cittaSedeLegale, capSedeLegale, provinciaSedeLegale, codiceNazioneSedeLegale, nazioneSedeLegale, partitaIva);
	}
	
	private static void setIndirizzo(Element partyElement, Element addressElement, String indirizzo, String civico, String citta, 
			String cap, String provincia, String codiceNazione, String nazione){
		Element cbcStreetName = new Element("StreetName").setNamespace(cbc);
		Element cbcBuildingNumber = new Element("BuildingNumber").setNamespace(cbc);
		Element cbcCityName = new Element("CityName").setNamespace(cbc);
		Element cbcPostalZone = new Element("PostalZone").setNamespace(cbc);
		Element cbcCountrySubentity = new Element("CountrySubentity").setNamespace(cbc);
		Element cacCountry = new Element("Country").setNamespace(cac);
		Element cbcIdentificationCode = new Element("IdentificationCode").setNamespace(cbc);
		Element cbcName_Country = new Element("Name").setNamespace(cbc);
		
		partyElement.addContent(addressElement);
		addressElement.addContent(cbcStreetName);
		cbcStreetName.setText(indirizzo);
		
		addressElement.addContent(cbcBuildingNumber);
		cbcBuildingNumber.setText(civico);
		
		addressElement.addContent(cbcCityName);
		cbcCityName.setText(citta);
		
		addressElement.addContent(cbcPostalZone);
		cbcPostalZone.setText(cap);
		
		addressElement.addContent(cbcCountrySubentity);
		cbcCountrySubentity.setText(provincia);
		
		if((codiceNazione != null && !codiceNazione.equals("")) && (nazione != null && !nazione.equals(""))) {
			addressElement.addContent(cacCountry);		
			cacCountry.addContent(cbcIdentificationCode);
			cbcIdentificationCode.setText(codiceNazione);
			cacCountry.addContent(cbcName_Country);
			cbcName_Country.setText(nazione);
		}
	}
	
	private static void setIndirizzo(Element partyElement, Element addressElement, String piano, String interno, String indirizzo, 
			String civico, String citta, String cap, String provincia, String codiceNazione, String nazione){
		Element cbcFloor = new Element("Floor").setNamespace(cbc);
		Element cbcRoom = new Element("Room").setNamespace(cbc);
		Element cbcStreetName = new Element("StreetName").setNamespace(cbc);
		Element cbcBuildingNumber = new Element("BuildingNumber").setNamespace(cbc);
		Element cbcCityName = new Element("CityName").setNamespace(cbc);
		Element cbcPostalZone = new Element("PostalZone").setNamespace(cbc);
		Element cbcCountrySubentity = new Element("CountrySubentity").setNamespace(cbc);
		Element cacCountry = new Element("Country").setNamespace(cac);
		Element cbcIdentificationCode = new Element("IdentificationCode").setNamespace(cbc);
		Element cbcName_Country = new Element("Name").setNamespace(cbc);
		
		partyElement.addContent(addressElement);
		
		if(piano != null && !piano.equals("")) {
			addressElement.addContent(cbcFloor);
			cbcFloor.setText(piano);
		}
		
		if(interno != null && !interno.equals("")) {
		addressElement.addContent(cbcRoom);
		cbcRoom.setText(interno);
		}
		
		addressElement.addContent(cbcStreetName);
		cbcStreetName.setText(indirizzo);
		
		addressElement.addContent(cbcBuildingNumber);
		cbcBuildingNumber.setText(civico);
		
		addressElement.addContent(cbcCityName);
		cbcCityName.setText(citta);
		
		addressElement.addContent(cbcPostalZone);
		cbcPostalZone.setText(cap);
		
		addressElement.addContent(cbcCountrySubentity);
		cbcCountrySubentity.setText(provincia);
		
		if((codiceNazione != null && !codiceNazione.equals("")) && (nazione != null && !nazione.equals(""))) {
			addressElement.addContent(cacCountry);		
			cacCountry.addContent(cbcIdentificationCode);
			cbcIdentificationCode.setText(codiceNazione);
			cacCountry.addContent(cbcName_Country);
			cbcName_Country.setText(nazione);
		}
	}
	
	private static void setDatiLegaliParte(Element cbcAccounting_Party, String registrazione,
											String indirizzoSedeLegale, String civicoSedeLegale, String cittaSedeLegale, String capSedeLegale,
											String provinciaSedeLegale, String codiceNazioneSedeLegale, String nazioneSedeLegale, String partitaIva){
		Element cacPartyLegalEntity = new Element("PartyLegalEntity").setNamespace(cac);
		Element cbcRegistrationName = new Element("RegistrationName").setNamespace(cbc);
		Element cbcCompanyID = new Element("CompanyID").setNamespace(cbc);
		Element cacRegistrationAddress  = new Element("RegistrationAddress").setNamespace(cac);
		
		cbcAccounting_Party.addContent(cacPartyLegalEntity);
		cacPartyLegalEntity.addContent(cbcRegistrationName);
		cbcRegistrationName.setText(registrazione);
		cacPartyLegalEntity.addContent(cbcCompanyID);
		cbcCompanyID.setText(partitaIva);
		//Sede legale
		if((indirizzoSedeLegale != null) && (!indirizzoSedeLegale.equals(""))){
			setIndirizzo(cacPartyLegalEntity, cacRegistrationAddress, indirizzoSedeLegale, civicoSedeLegale, cittaSedeLegale, capSedeLegale, 
					provinciaSedeLegale, codiceNazioneSedeLegale, nazioneSedeLegale);
		}
	}

	/**
	 * Aggiunge il tag <i>cac:LegalMonetaryTotal</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param taxDetail 
	 * @param codiceIva 
	 * @param numeroRiga 
	 * @param quantita 
	 * @param valoreNormale 
	 * @param indicatoreSPA 
	 * @param ammontareSPA 
	 * @param ragioneSPA 
	 * @param totaleRiga 
	 * @param descrizione 
	 * @param unitaDiMisura 
	 * @param valuta 
	 * 
	 * @return (void)
	 */
	public static void setRigaFattura(Element root, String itemKeywordValue, String itemNameValue, int lineNumber,
			String lineAccountingCost, String orderLineReference, String sellersItemIdentification,
			double invoicedQuantity, double priceAmount,
			List<AllowanceCharge> allowanceCharges, double lineExtensionAmount,
			TaxDetail taxDetail, String codiceIva, String itemDescription, String unitCode, String currencyID, 
			PreviousDocument contract, List<String> note, String contatore){
		
		Element cacInvoiceLine = new Element("InvoiceLine").setNamespace(cac);
		Element cbcID = new Element("ID").setNamespace(cbc);
		Element cbcInvoicedQuantity = new Element("InvoicedQuantity").setNamespace(cbc);
		Element cacPrice = new Element("Price").setNamespace(cac);
		Element cbcPriceAmount = new Element("PriceAmount").setNamespace(cbc);


		Element cbcLineExtensionAmount = new Element("LineExtensionAmount").setNamespace(cbc);
		
		Element cbcAccountingCost = new Element("AccountingCost").setNamespace(cbc);
		
		Element cacOrderLineReference = new Element("OrderLineReference").setNamespace(cac);
		Element cbcOrderLineReferenceID = new Element("LineID").setNamespace(cbc);
		
		Element cacItem = new Element("Item").setNamespace(cac);
		Element cbcDescription = new Element("Description").setNamespace(cbc);
		Element cbcKeyword = new Element("Keyword").setNamespace(cbc);		
		Element cbcName = new Element("Name").setNamespace(cbc);
		Element cacSellersItemIdentification = new Element("SellersItemIdentification").setNamespace(cac);
		Element cbcSellersItemIdentificationID = new Element("ID").setNamespace(cbc);
		
		Element cacClassifiedTaxCategory = new Element("ClassifiedTaxCategory").setNamespace(cac);
		
		Element cacDocumentReference_Contratto = new Element("DocumentReference").setNamespace(cac);
		Element documentReferenceCbcID_Contratto = new Element("ID").setNamespace(cbc);
		Element documentReferenceCbcDocumentType_Contratto = new Element("DocumentType").setNamespace(cbc);
		
		Element cacDocumentReference_Contatore = new Element("DocumentReference").setNamespace(cac);
		Element documentReferenceCbcID_Contatore = new Element("ID").setNamespace(cbc);
		Element documentReferenceCbcDocumentType_Contatore = new Element("DocumentType").setNamespace(cbc);

		root.addContent(cacInvoiceLine);
		
		//ID
		cacInvoiceLine.addContent(cbcID);
		cbcID.setText(Integer.toString(lineNumber));
		
		//Note
		if((note != null) && (note.size() > 0)){
			for(int y = 0; y < note.size(); y++){
				setNote(cacInvoiceLine, note.get(y));
			}
		}
		
		//InvoicedQuantity
		if(invoicedQuantity != 0) {
			cacInvoiceLine.addContent(cbcInvoicedQuantity);
			if(unitCode != null && !unitCode.equals("")) {
				cbcInvoicedQuantity.setAttribute("unitCode", unitCode.toUpperCase()); //TODO NES CodeList
				//cbcInvoicedQuantity.setAttribute(new Attribute("schemeID", "UN/ECE rec 20"));
			}
			cbcInvoicedQuantity.setText(Double.toString(invoicedQuantity));
		}
		
		//LineExtensionAmount
		cacInvoiceLine.addContent(cbcLineExtensionAmount);
		cbcLineExtensionAmount.setAttribute("currencyID", currencyID);
		cbcLineExtensionAmount.setText(Double.toString(lineExtensionAmount));
		
		//LineAccountingCost
		if(lineAccountingCost != null){
			cacInvoiceLine.addContent(cbcAccountingCost);
			cbcAccountingCost.setText(lineAccountingCost);
		}
		
		//OrderLineReference
		if(orderLineReference != null){
			cacInvoiceLine.addContent(cacOrderLineReference);
			cacOrderLineReference.addContent(cbcOrderLineReferenceID);
			cbcOrderLineReferenceID.setText(orderLineReference);
		}
		
		//FreeOfChargeIndicator
		
		
		//DocumentReferences
		//Le righe di dettaglio aggiuntive non hanno il contratto di riferimento
		if((contract != null) && (contract.getDocumentCode() != null)) {
			cacInvoiceLine.addContent(cacDocumentReference_Contratto);
			cacDocumentReference_Contratto.addContent(documentReferenceCbcID_Contratto);
			documentReferenceCbcID_Contratto.setText(contract.getDocumentCode());
			cacDocumentReference_Contratto.addContent(documentReferenceCbcDocumentType_Contratto);
			documentReferenceCbcDocumentType_Contratto.setText(contract.getDocumentType());
		}
		if((contatore != null) && (!contatore.equals(""))) {
			cacInvoiceLine.addContent(cacDocumentReference_Contatore);
			cacDocumentReference_Contatore.addContent(documentReferenceCbcID_Contatore);
			documentReferenceCbcID_Contatore.setText(contatore);
			cacDocumentReference_Contatore.addContent(documentReferenceCbcDocumentType_Contatore);
			documentReferenceCbcDocumentType_Contatore.setText("contatore");
		}
		
		//AllowanceCharge
		if(allowanceCharges != null){			
			for(int i = 0; i < allowanceCharges.size(); i++){
				Element cacAllowanceCharge = new Element("AllowanceCharge").setNamespace(cac);
				Element cbcChargeIndicator = new Element("ChargeIndicator").setNamespace(cbc);
				Element cbcAllowanceChargeReason = new Element("AllowanceChargeReason").setNamespace(cbc);
				Element cbcAmount = new Element("Amount").setNamespace(cbc);
								
				cacPrice.addContent(cacAllowanceCharge);
				cacAllowanceCharge.addContent(cbcChargeIndicator);
				if(allowanceCharges.get(i).getAmount() > 0){
					cbcChargeIndicator.setText("true");
				}
				else {
					cbcChargeIndicator.setText("false");
				}
		
				cacAllowanceCharge.addContent(cbcAllowanceChargeReason);
				cbcAllowanceChargeReason.setText(allowanceCharges.get(i).getAllowanceChargeReason());
				
				cacAllowanceCharge.addContent(cbcAmount);
				cbcAmount.setAttribute("currencyID", currencyID);
				cbcAmount.setText(Double.toString(allowanceCharges.get(i).getAmount()));
			}
		}
		
		//TaxTotal
		if(taxDetail != null){
			setRiepilogoIva(cacInvoiceLine, codiceIva, taxDetail.getTaxPercent(), String.valueOf(taxDetail.getTaxableAmount()), String.valueOf(taxDetail.getTaxAmount()), false);
		}

		//Item
		cacInvoiceLine.addContent(cacItem);
		
		if((itemDescription != null) && (!itemDescription.equals(""))) {	
			cacItem.addContent(cbcDescription);
			cbcDescription.setText(itemDescription);
		}
		
		cacItem.addContent(cbcName);
		cbcName.setText(itemNameValue);
		
		if((itemKeywordValue == null) || (itemKeywordValue.equals(""))) {
			itemKeywordValue = itemKeyword.RIGA.toString();
		}
		cacItem.addContent(cbcKeyword);
		cbcKeyword.setText(itemKeywordValue);
		
		//SellersItemIdentification
		if((sellersItemIdentification != null) && (!sellersItemIdentification.equals(""))){
			cacItem.addContent(cacSellersItemIdentification);
			cacSellersItemIdentification.addContent(cbcSellersItemIdentificationID);
			cbcSellersItemIdentificationID.setText(sellersItemIdentification);
		}
		
		//Dettaglio tasse (in Item)
		if(taxDetail != null){
			cacItem.addContent(cacClassifiedTaxCategory);		
			setTaxCategory(cacClassifiedTaxCategory, codiceIva, taxDetail.getTaxPercent(), null);
		}
		
		//Price
		if(priceAmount != 0) {
			cacInvoiceLine.addContent(cacPrice);
			cacPrice.addContent(cbcPriceAmount);
			cbcPriceAmount.setAttribute("currencyID", currencyID);
			cbcPriceAmount.setText(Double.toString(priceAmount));
			if(allowanceCharges != null){			
				for(int i = 0; i < allowanceCharges.size(); i++){
					Element cacAllowanceCharge = new Element("AllowanceCharge").setNamespace(cac);
					Element cbcChargeIndicator = new Element("ChargeIndicator").setNamespace(cbc);
					Element cbcAllowanceChargeReason = new Element("AllowanceChargeReason").setNamespace(cbc);
					Element cbcAmount = new Element("Amount").setNamespace(cbc);

					cacPrice.addContent(cacAllowanceCharge);
					cacAllowanceCharge.addContent(cbcChargeIndicator);
					if(allowanceCharges.get(i).getAmount() > 0){
						cbcChargeIndicator.setText("true");
					}
					else {
						cbcChargeIndicator.setText("false");
					}

					cacAllowanceCharge.addContent(cbcAllowanceChargeReason);
					cbcAllowanceChargeReason.setText(allowanceCharges.get(i).getAllowanceChargeReason());

					cacAllowanceCharge.addContent(cbcAmount);
					cbcAmount.setAttribute("currencyID", currencyID);
					cbcAmount.setText(Double.toString(allowanceCharges.get(i).getAmount()));				
				}
			}
		}
	}
	
	/**
	 * Aggiunge il tag <i>cac:LegalMonetaryTotal</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param totaleFattura
	 * 
	 * @return (void)
	 */
	public static void setTotaleFattura(Element root, String totaleFattura, String currency, double lineExtensionAmount,
			double taxExclusiveAmount, double taxInclusiveAmount, double allowanceTotalAmount, double chargeTotalAmount){
		Element cacLegalMonetaryTotal = new Element("LegalMonetaryTotal").setNamespace(cac);
		Element cbcPayableAmount = new Element("PayableAmount").setNamespace(cbc);
		cbcPayableAmount.setAttribute("currencyID", currency);
		
		root.addContent(cacLegalMonetaryTotal);
		
		if(lineExtensionAmount != 0) {
			Element cbcLineExtensionAmount = new Element("LineExtensionAmount").setNamespace(cbc);
			cacLegalMonetaryTotal.addContent(cbcLineExtensionAmount);
			cbcLineExtensionAmount.setAttribute("currencyID", currency);
			cbcLineExtensionAmount.setText(""+lineExtensionAmount);
		}
		
		if(taxExclusiveAmount != 0) {
			Element cbcTaxExclusiveAmount = new Element("TaxExclusiveAmount").setNamespace(cbc);
			cacLegalMonetaryTotal.addContent(cbcTaxExclusiveAmount);
			cbcTaxExclusiveAmount.setAttribute("currencyID", currency);
			cbcTaxExclusiveAmount.setText(""+taxExclusiveAmount);
		}

		if(taxInclusiveAmount != 0) {
			Element cbcTaxInclusiveAmount = new Element("TaxInclusiveAmount").setNamespace(cbc);
			cacLegalMonetaryTotal.addContent(cbcTaxInclusiveAmount);
			cbcTaxInclusiveAmount.setAttribute("currencyID", currency);
			cbcTaxInclusiveAmount.setText(""+taxInclusiveAmount);
		}
		
		if(allowanceTotalAmount != 0) {
			Element cbcAllowanceTotalAmount = new Element("AllowanceTotalAmount").setNamespace(cbc);
			cacLegalMonetaryTotal.addContent(cbcAllowanceTotalAmount);
			cbcAllowanceTotalAmount.setAttribute("currencyID", currency);
			cbcAllowanceTotalAmount.setText(""+allowanceTotalAmount);
		}
		
		if(chargeTotalAmount != 0) {
			Element cbcChargeTotalAmount = new Element("ChargeTotalAmount").setNamespace(cbc);
			cacLegalMonetaryTotal.addContent(cbcChargeTotalAmount);
			cbcChargeTotalAmount.setAttribute("currencyID", currency);
			cbcChargeTotalAmount.setText(""+chargeTotalAmount);
		}
		
		cacLegalMonetaryTotal.addContent(cbcPayableAmount);		
		cbcPayableAmount.setText(totaleFattura);
	}
	
	public static void setTotaleFattura(Element root, String totaleFattura, String currency){
		Element cacLegalMonetaryTotal = new Element("LegalMonetaryTotal").setNamespace(cac);
		Element cbcPayableAmount = new Element("PayableAmount").setNamespace(cbc);
		cbcPayableAmount.setAttribute("currencyID", currency);
		Element cbcLineExtensionAmount = new Element("LineExtensionAmount").setNamespace(cbc);
		cbcLineExtensionAmount.setAttribute("currencyID", currency);
		Element cbcTaxExclusiveAmount = new Element("TaxExclusiveAmount").setNamespace(cbc);
		cbcTaxExclusiveAmount.setAttribute("currencyID", currency);
		Element cbcTaxInclusiveAmount = new Element("TaxInclusiveAmount").setNamespace(cbc);
		cbcTaxInclusiveAmount.setAttribute("currencyID", currency);
		
		root.addContent(cacLegalMonetaryTotal);
		cacLegalMonetaryTotal.addContent(cbcLineExtensionAmount);
		cacLegalMonetaryTotal.addContent(cbcTaxExclusiveAmount);
		cacLegalMonetaryTotal.addContent(cbcTaxInclusiveAmount);
		
		cacLegalMonetaryTotal.addContent(cbcPayableAmount);		
		cbcPayableAmount.setText(totaleFattura);
	}
	
	/**
	 * Aggiunge il tag <i>cac:TaxTotal</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param totaleFattura
	 * 
	 * @return Element cacTaxTotal da utilizzare per agganciare eventuali subtotali
	 */
	public static Element setRiepilogoIva(Element root, String codiceIva, Double aliquotaIva, String imponibile, String importoIva, Boolean subTotal){	
		
		if(!subTotal) {
			Element cacTaxTotal = new Element("TaxTotal").setNamespace(cac);
			Element cbcTaxAmount = new Element("TaxAmount").setNamespace(cbc);
			//Element cbcRoundingAmount = new Element("RoundingAmount").setNamespace(cbc);
			
			root.addContent(cacTaxTotal);
		
			cacTaxTotal.addContent(cbcTaxAmount);
			cbcTaxAmount.setAttribute(new Attribute("currencyID","EUR"));
			cbcTaxAmount.setText(importoIva);
		
//			cacTaxTotal.addContent(cbcRoundingAmount);
//			cbcRoundingAmount.setText("");
			
//			Element cacTaxSubtotal = new Element("TaxSubtotal").setNamespace(cac);
//			Element cbcTaxAmount_Subtotale = new Element("TaxAmount").setNamespace(cbc);
//			Element cacTaxCategory = new Element("TaxCategory").setNamespace(cac);
//			
//			cacTaxTotal.addContent(cacTaxSubtotal);
//			
//			Element cbcTaxableAmount = new Element("TaxableAmount").setNamespace(cbc);
//			cacTaxSubtotal.addContent(cbcTaxableAmount);
//			cbcTaxableAmount.setAttribute(new Attribute("currencyID","EUR"));
//			cbcTaxableAmount.setText(imponibile);
//			
//			cacTaxSubtotal.addContent(cbcTaxAmount_Subtotale);
//			cbcTaxAmount_Subtotale.setAttribute(new Attribute("currencyID","EUR"));
//			cbcTaxAmount_Subtotale.setText(importoIva);
//			
//			cacTaxSubtotal.addContent(cacTaxCategory);
//			setTaxCategory(cacTaxCategory, codiceIva, aliquotaIva, null);
			
			
			return cacTaxTotal;
		}
		else {
			Element cacTaxSubtotal = new Element("TaxSubtotal").setNamespace(cac);
			Element cbcTaxAmount_Subtotale = new Element("TaxAmount").setNamespace(cbc);
			Element cacTaxCategory = new Element("TaxCategory").setNamespace(cac);
			
			root.addContent(cacTaxSubtotal);
			
			Element cbcTaxableAmount = new Element("TaxableAmount").setNamespace(cbc);
			cacTaxSubtotal.addContent(cbcTaxableAmount);
			cbcTaxableAmount.setAttribute(new Attribute("currencyID","EUR"));
			cbcTaxableAmount.setText(imponibile);
			
			cacTaxSubtotal.addContent(cbcTaxAmount_Subtotale);
			cbcTaxAmount_Subtotale.setAttribute(new Attribute("currencyID","EUR"));
			cbcTaxAmount_Subtotale.setText(importoIva);
			
			cacTaxSubtotal.addContent(cacTaxCategory);
			setTaxCategory(cacTaxCategory, codiceIva, aliquotaIva, null);
			
			return null;
		}
	}
	
	/**
	 * Aggiunge il tag <i>cac:PaymentMeans</i> e <i>cac:PaymentTerms</i>
	 * 
	 * @param dataScadenza (String)
	 * @param note (String)
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	
	public static void setDataScadenza(Element root, String dataScadenza, String note) {
		Element cacPaymentMeans = new Element("PaymentMeans").setNamespace(cac);
		Element cacPaymentDueDate = new Element("PaymentDueDate").setNamespace(cbc);
		Element cbcPaymentMeansCode = new Element("PaymentMeansCode").setNamespace(cbc);
		
		Element cacPaymentTerms = new Element("PaymentTerms").setNamespace(cac);
		Element cbcNote = new Element("Note").setNamespace(cbc);

		//TODO: gestire i possibili payment means codes
		//per ora è impostato con codice "Instrument not defined" secondo le NES codelists
		//http://www.esv.se/download/18.6dae77a0113497f158680002577/NES+Code+Lists+and+Identification+Schemes+-+Version+2.pdf		
		cbcPaymentMeansCode.setAttribute(new Attribute("listID", "UN/ECE 4461 Subset"));
		cbcPaymentMeansCode.setAttribute(new Attribute("listAgencyID","NES"));
		cbcPaymentMeansCode.addContent("1");// 1 è il codice corrispondente a "Nessun mezzo di pagamento specificato" 
		cacPaymentMeans.addContent(cbcPaymentMeansCode);
		
		root.addContent(cacPaymentMeans);
				
		//Data di scadenza pagamento
		if((dataScadenza != null) && (!dataScadenza.equals(""))){
			cacPaymentMeans.addContent(cacPaymentDueDate);
			cacPaymentDueDate.setText(formatDateString(dataScadenza));
		}
				
		//Note
		if((note != null) && (!note.equals(""))){
			root.addContent(cacPaymentTerms);
			cacPaymentTerms.addContent(cbcNote);
			cacPaymentDueDate.setText(formatDateString(dataScadenza));			
		}
	}

	
	/**
	 * Aggiunge il tag <i>cac:Delivery</i>
	 * 
	 * @param dataScadenza (String)
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	
	public static void setFornitura(Element root, String codiceContatore, String codiceContratto,
			String indirizzoFornitura, String civicoFornitura,
			String capFornitura, String localitaFornitura, String provinciaFornitura) {
		setFornitura(root, codiceContatore, null, codiceContratto, indirizzoFornitura, civicoFornitura, capFornitura, localitaFornitura, provinciaFornitura);
	}
	
	public static void setFornitura(Element root, String codiceContatore, String POD, String codiceContratto,
			String indirizzoFornitura, String civicoFornitura,
			String capFornitura, String localitaFornitura, String provinciaFornitura) {

		Element cacDelivery = new Element("Delivery").setNamespace(cac);
		Element cbcID = new Element("ID").setNamespace(cbc);
		Element cbcTrackingID = new Element("TrackingID").setNamespace(cbc);
		Element cacDeliveryAddress = new Element("DeliveryAddress").setNamespace(cac);
		//TODO: vedere se va bene il nome cbc:PointOfDelivery
		//Element cbcPOD = new Element("PointOfDeliveryID").setNamespace(cbc);
		
		root.addContent(cacDelivery);
		
		cacDelivery.addContent(cbcID);
		cbcID.setText(codiceContatore);
		
//		if(POD == null) POD="";
//		cacDelivery.addContent(cbcPOD);
//		cbcPOD.setText(POD);
		
		
		cacDelivery.addContent(cbcTrackingID);
		cbcTrackingID.setText(codiceContratto);
		
		setIndirizzo(cacDelivery, cacDeliveryAddress, indirizzoFornitura, civicoFornitura, localitaFornitura, capFornitura, provinciaFornitura, "IT", "Italia");
	}

	
	/**
	 * Aggiunge il tag <i>cbc:Note</i>
	 * 
	 * @param note (String)
	 * @param element (Element) elemento XML a cui aggiungere il tag
	 * 
	 * @return (void)
	 */
	public static void setNote(Element element, String note) {

		if(note != null && !note.equals("")) {
			Element cbcNote = new Element("Note").setNamespace(cbc);

			element.addContent(cbcNote);
			cbcNote.setText(note);
		}
	}
	
	public static void setTaxCategory(Element element, String id, Double percent, String name){
		//TODO da gestire la casistica completa
		//UN/CEFACT Code List 5305 “Duty/Tax/Fee Category Code” release 06B
		//http://www.unece.org/trade/untdid/d06b/tred/tred5305.htm
		//http://www.esv.se/download/18.6dae77a0113497f158680002577/NES+Code+Lists+and+Identification+Schemes+-+Version+2.pdf
		
		Element taxCategoryCbcID = new Element("ID").setNamespace(cbc);
		Element taxCategoryCbcName = new Element("Name").setNamespace(cbc);
		String taxCategoryID = id;
		String taxCategoryPercent;
		
		
		if(percent == null){
			if(id == null) taxCategoryID = "G";
			taxCategoryPercent = "";
		}
		else if(percent == 0){
			if(id == null) taxCategoryID = "Z";
			taxCategoryPercent = "0";
		}
		else {
			if(id == null) taxCategoryID = "S";
			taxCategoryPercent = String.valueOf(percent);
		}
		
		element.addContent(taxCategoryCbcID);
		taxCategoryCbcID.setAttribute(new Attribute("schemeID", "UN/ECE 5305"));
		taxCategoryCbcID.setAttribute(new Attribute("schemeAgencyID","6"));
		taxCategoryCbcID.setAttribute(new Attribute("schemeAgencyName","United Nations Economic Commission for Europe"));
		taxCategoryCbcID.setAttribute(new Attribute("schemeName","Tax Category Identifier"));
		taxCategoryCbcID.setAttribute(new Attribute("schemeVersionID","D06B"));
		taxCategoryCbcID.setText(taxCategoryID);
		
		if(name != null){
			element.addContent(taxCategoryCbcName);
			taxCategoryCbcName.setText(name);
		}
		
		if(taxCategoryPercent != null && !taxCategoryPercent.equals("")) {
			Element taxCategoryCbcPercent = new Element("Percent").setNamespace(cbc);
			element.addContent(taxCategoryCbcPercent);
			taxCategoryCbcPercent.setText(taxCategoryPercent);
		}
		
		setTaxScheme(element);
	}
	
	public static void setTaxScheme(Element element){
		//TODO da gestire la casistica completa
		//UN/CEFACT Code List 5153 “Duty or tax or fee type name code” release 06B
		//http://www.unece.org/trade/untdid/d06b/tred/tred5153.htm
		//http://www.esv.se/download/18.6dae77a0113497f158680002577/NES+Code+Lists+and+Identification+Schemes+-+Version+2.pdf
		
		Element cacTaxScheme = new Element("TaxScheme").setNamespace(cac);
		Element taxSchemeCbcID = new Element("ID").setNamespace(cbc);

		element.addContent(cacTaxScheme);
		
		cacTaxScheme.addContent(taxSchemeCbcID);
		taxSchemeCbcID.setAttribute(new Attribute("schemeID", "UN/ECE 5153"));
		taxSchemeCbcID.setAttribute(new Attribute("schemeAgencyID","6"));
		taxSchemeCbcID.setAttribute(new Attribute("schemeAgencyName","United Nations Economic Commission for Europe"));
		taxSchemeCbcID.setAttribute(new Attribute("schemeName","Tax Category Identifier"));
		taxSchemeCbcID.setAttribute(new Attribute("schemeVersionID","D06B"));
		taxSchemeCbcID.setText("VAT");
	}
	
	/**  TAG RELATIVI ALL'UTILITY STATEMENT  **/
	
	/**
	 * Aggiunge il tag <i>ext:UBLExtensions</i>
	 * 
	 * @param root (Element) root del doc XML a cui aggiungere il tag
	 * @param id (String) identificativo documento
	 * @param issueDate (String) data di emissione
	 * @param typeCode (String) tipo di report
	 * 
	 * @return (Element) utilityStatement
	 */
	public static Element setUtilityStatement(Element root, String id, String issueDate, String typeCode) {

		Element extUblExtensions = new Element("UBLExtensions").setNamespace(ext);
		Element extUblExtension = new Element("UBLExtension").setNamespace(ext);
		Element extExtensionContent = new Element("ExtensionContent").setNamespace(ext);
		Element utilityStatement = new Element("UtilityStatement");
		
		root.addContent(extUblExtensions);
		extUblExtensions.addContent(extUblExtension);
		extUblExtension.addContent(extExtensionContent);
		extExtensionContent.addContent(utilityStatement);
	
		utilityStatement.addNamespaceDeclaration(xsi);
		utilityStatement.setAttribute("schemaLocation", "urn:oasis:names:specification:ubl:schema:xsd:UtilityStatement-2 http://docs.oasis-open.org/ubl/prd2-UBL-2.1/xsd/maindoc/UBL-UtilityStatement-2.1.xsd", xsi);
		utilityStatement.addNamespaceDeclaration(cbc);
		utilityStatement.addNamespaceDeclaration(cac);
		utilityStatement.addNamespaceDeclaration(usCcts);
		utilityStatement.addNamespaceDeclaration(ext);
		utilityStatement.setNamespace(utilityStatementNamespace);
		
		Element cbcId = new Element("ID").setNamespace(cbc);
		Element cbcIssueDate = new Element("IssueDate").setNamespace(cbc);
		Element cbcTypeCode = new Element("UtilityStatementTypeCode").setNamespace(cbc);
		
		utilityStatement.addContent(cbcId);
		cbcId.setText(id);
		if((issueDate != null) && (!issueDate.equals(""))){
			utilityStatement.addContent(cbcIssueDate);
			cbcIssueDate.setText(formatDateString(issueDate));
		}
		utilityStatement.addContent(cbcTypeCode);
		cbcTypeCode.setText(typeCode);
		
		return utilityStatement;
	}

	/**
	 * Aggiunge il tag <i>cac:ParentDocumentReference</i>
	 *
	 * @param utilityStatement (Element) utilityStatement del doc XML a cui aggiungere il tag 
	 * @param id (String) numero della bolletta
	 * 
	 * @return (void)
	 */
	public static void setUSParentDocumentReference(Element utilityStatement, String id) {
		Element cacParentDocumentReference = new Element("ParentDocumentReference").setNamespace(cac);
		Element cbcId = new Element("ID").setNamespace(cbc);
		utilityStatement.addContent(cacParentDocumentReference);
		cacParentDocumentReference.addContent(cbcId);
		cbcId.setText(id);
	}

	/**
	 * Aggiunge il tag <i>cac:SenderParty</i>
	 *
	 * @param utilityStatement (Element) utilityStatement del doc XML a cui aggiungere il tag 
	 * @param name (String) ragione sociale della societa'
	 * @param streetName (String) via
	 * @param buildingNumber (String) numero civico
	 * @param cityName (String) localita'
	 * @param postalZone (String) cap
	 * @param countrySubentity (String) provincia
	 * @param countryId (String) id della nazione (ad esempio IT)
	 * @param countryName (String) nome della nazione
	 * 
	 * @return (void)
	 */
	public static void setUSSenderParty(Element utilityStatement, String name, String streetName, String buildingNumber,
			String cityName, String postalZone, String countrySubentity, String countryId, String countryName) {
		Element cacSenderParty = new Element("SenderParty").setNamespace(cac);
		Element cacPartyName = new Element("PartyName").setNamespace(cac);
		Element cbcName = new Element("Name").setNamespace(cbc);
		Element cacPostalAddress = new Element("PostalAddress").setNamespace(cac);

		utilityStatement.addContent(cacSenderParty);
		cacSenderParty.addContent(cacPartyName);
		cacPartyName.addContent(cbcName);
		cbcName.setText(name);

		setIndirizzo(cacSenderParty, cacPostalAddress, streetName, buildingNumber, cityName, postalZone, countrySubentity, countryId, countryName);

	}

	/**
	 * Aggiunge il tag <i>cac:ReceiverParty</i>
	 *
	 * @param utilityStatement (Element) utilityStatement del doc XML a cui aggiungere il tag 
	 * @param name (String) denominazione
	 * @param streetName (String) via
	 * @param buildingNumber (String) numero civico
	 * @param cityName (String) localita'
	 * @param postalZone (String) cap
	 * @param countrySubentity (String) provincia
	 * @param countryId (String) id della nazione (ad esempio IT)
	 * @param countryName (String) nome della nazione
	 * 
	 * @return (void)
	 */
	public static void setUSReceiverParty(Element utilityStatement, String name, String streetName, String buildingNumber,
			String cityName, String postalZone, String countrySubentity, String countryId, String countryName) {
		Element cacReceiverParty = new Element("ReceiverParty").setNamespace(cac);
		Element cacPartyName = new Element("PartyName").setNamespace(cac);
		Element cbcName = new Element("Name").setNamespace(cbc);
		Element cacPostalAddress = new Element("PostalAddress").setNamespace(cac);

		utilityStatement.addContent(cacReceiverParty);
		cacReceiverParty.addContent(cacPartyName);
		cacPartyName.addContent(cbcName);
		cbcName.setText(name);

		setIndirizzo(cacReceiverParty, cacPostalAddress, streetName, buildingNumber, cityName, postalZone, countrySubentity, countryId, countryName);

	}

	/**
	 * Aggiunge il tag <i>cac:SubscriberParty</i>
	 *
	 * @param utilityStatement (Element) utilityStatement del doc XML a cui aggiungere il tag 
	 * @param name (String) denominazione
	 * @param streetName (String) via
	 * @param buildingNumber (String) numero civico
	 * @param cityName (String) localita'
	 * @param postalZone (String) cap
	 * @param countrySubentity (String) provincia
	 * @param countryId (String) id della nazione (ad esempio IT)
	 * @param countryName (String) nome della nazione
	 * 
	 * @return (void)
	 */
	public static void setUSSubscriberParty(Element utilityStatement, String name, String streetName, String buildingNumber,
			String cityName, String postalZone, String countrySubentity, String countryId, String countryName) {
		Element cacSubscriberParty = new Element("SubscriberParty").setNamespace(cac);
		Element cacPartyName = new Element("PartyName").setNamespace(cac);
		Element cbcName = new Element("Name").setNamespace(cbc);
		Element cacPostalAddress = new Element("PostalAddress").setNamespace(cac);

		utilityStatement.addContent(cacSubscriberParty);
		cacSubscriberParty.addContent(cacPartyName);
		cacPartyName.addContent(cbcName);
		cbcName.setText(name);

		setIndirizzo(cacSubscriberParty, cacPostalAddress, streetName, buildingNumber, cityName, postalZone, countrySubentity, countryId, countryName);

	}
	
	/**
	 * Aggiunge il tag <i>cac:SubscriberConsumption</i>
	 *
	 * @param utilityStatement (Element) utilityStatement del doc XML a cui aggiungere il tag 
	 * @param specificationTypeCode (String) tipo fornitura
	 * @param note (String) altre informazioni relative alla fornitura e/o all'impianto di distribuzione
	 * 
	 * @return (Element) cacSubscriberConsumption
	 */
	public static Element setUSSubscriberConsumption(Element utilityStatement, String specificationTypeCode, String note) {
		Element cacSubscriberConsumption = new Element("SubscriberConsumption").setNamespace(cac);
		Element cbcSpecificationTypeCode = new Element("SpecificationTypeCode").setNamespace(cbc);
		Element cbcNote = new Element("Note").setNamespace(cbc);

		utilityStatement.addContent(cacSubscriberConsumption);
		cacSubscriberConsumption.addContent(cbcSpecificationTypeCode);
		cbcSpecificationTypeCode.setText(specificationTypeCode);
		if(note != null && !note.equals("")) {
			cacSubscriberConsumption.addContent(cbcNote);
			cbcNote.setText(note);
		}
		
		return cacSubscriberConsumption;
	}
	
	/**
	 * Aggiunge il tag <i>cac:UtilityConsumptionPoint</i>
	 *
	 * @param subscriberConsumption (Element) subscriberConsumption del doc XML a cui aggiungere il tag 
	 * @param id (String) identificativo univoco del punto di fornitura
	 * @param floor (String) piano
	 * @param room (String) numero interno
	 * @param streetName (String) via
	 * @param buildingNumber (String) numero civico
	 * @param cityName (String) localita'
	 * @param postalZone (String) cap
	 * @param countrySubentity (String) provincia
	 * @param countryId (String) id della nazione (ad esempio IT)
	 * @param countryName (String) nome della nazione
	 * 
	 * @return (Element) cacUtilityConsumptionPoint nodo a cui aggiungere gli UtilityMeter 
	 */
	public static Element setUSUtilityConsumptionPoint(Element subscriberConsumption, String id, String floor,
			String room, String streetName, String buildingNumber, String cityName, String postalZone, 
			String countrySubentity, String countryId, String countryName) {
		Element cacUtilityConsumptionPoint = new Element("UtilityConsumptionPoint").setNamespace(cac);
		Element cbcId = new Element("ID").setNamespace(cbc);
		subscriberConsumption.addContent(cacUtilityConsumptionPoint);
		cacUtilityConsumptionPoint.addContent(cbcId);
		cbcId.setText(id);
		
		Element cacAddress = new Element("Address").setNamespace(cac);
		
		setIndirizzo(cacUtilityConsumptionPoint, cacAddress, floor, room, streetName, buildingNumber, cityName, postalZone, countrySubentity, countryId, countryName);
		
		return cacUtilityConsumptionPoint;
	}
	
	/**
	 * Aggiunge il tag <i>cac:UtilityMeter</i>
	 *
	 * @param cacUtilityConsumptionPoint (Element) UtilityConsumptionPoint del doc XML a cui aggiungere il tag 
	 * @param meterNumber (String) identificativo del contatore
	 * 
	 * @return (Element) cacUtilityMeter nodo a cui aggiungere i MeterReading 
	 */
	public static Element setUSUtilityMeter(Element cacUtilityConsumptionPoint, String meterNumber) {
		Element cacUtilityMeter = new Element("UtilityMeter").setNamespace(cac);
		Element cbcMeterNumber = new Element("MeterNumber").setNamespace(cbc);

		cacUtilityConsumptionPoint.addContent(cacUtilityMeter);
		cacUtilityMeter.addContent(cbcMeterNumber);
		cbcMeterNumber.setText(meterNumber);
		
		return cacUtilityMeter;
	}
	
	/**
	 * Aggiunge il tag <i>cac:MeterReading</i>
	 *
	 * @param cacUtilityMeter (Element) UtilityMeter del doc XML a cui aggiungere il tag 
	 * @param id (String) identificativo della lettura
	 * @param previousMeterReadingDate (String) data lettura precedente
	 * @param previousMeterQuantity (double) valore precedente
	 * @param latestMeterReadingDate (String) data lettura attuale
	 * @param latestMeterQuantity (double) valore attuale
	 * @param previousMeterReadingMethodCode (String) metodo lettura precedente
	 * @param latestMeterReadingMethodCode (String) metodo lettura attuale
	 * @param deliveredQuantity (double) quantita' fornita
	 * @param unitCode (String) unita' di misura riferita alla quantita'
	 * 
	 * @return (void)
	 */
	public static void setUSMeterReading(Element cacUtilityMeter, String id, String previousMeterReadingDate, double previousMeterQuantity,
			String latestMeterReadingDate, double latestMeterQuantity, String previousMeterReadingMethodCode, 
			String latestMeterReadingMethodCode, double deliveredQuantity, String unitCode) {
		Element cacMeterReading = new Element("MeterReading").setNamespace(cac);
		Element cbcId = new Element("ID").setNamespace(cbc);
		Element cbcPreviousMeterReadingDate = new Element("PreviousMeterReadingDate").setNamespace(cbc);
		Element cbcPreviousMeterQuantity = new Element("PreviousMeterQuantity").setNamespace(cbc);
		Element cbcLatestMeterReadingDate = new Element("LatestMeterReadingDate").setNamespace(cbc);
		Element cbcLatestMeterQuantity = new Element("LatestMeterQuantity").setNamespace(cbc);
		Element cbcPreviousMeterReadingMethodCode = new Element("PreviousMeterReadingMethodCode").setNamespace(cbc);
		Element cbcLatestMeterReadingMethodCode = new Element("LatestMeterReadingMethodCode").setNamespace(cbc);
		Element cbcDeliveredQuantity = new Element("DeliveredQuantity").setNamespace(cbc);

		cacUtilityMeter.addContent(cacMeterReading);
		cacMeterReading.addContent(cbcId);
		cbcId.setText(id);
		
		if((previousMeterReadingDate != null) && (!previousMeterReadingDate.equals(""))){
			cacMeterReading.addContent(cbcPreviousMeterReadingDate);
			cbcPreviousMeterReadingDate.setText(formatDateString(previousMeterReadingDate));
		}
		
		cbcPreviousMeterQuantity.setAttribute(new Attribute("unitCode", unitCode));
		//cbcPreviousMeterQuantity.setAttribute(new Attribute("schemeID", "UN/ECE rec 20"));
		cbcPreviousMeterQuantity.setText("" + previousMeterQuantity);
		cacMeterReading.addContent(cbcPreviousMeterQuantity);
		
		if((latestMeterReadingDate != null) && (!latestMeterReadingDate.equals(""))){
			cacMeterReading.addContent(cbcLatestMeterReadingDate);
			cbcLatestMeterReadingDate.setText(formatDateString(latestMeterReadingDate));
		}

		cbcLatestMeterQuantity.setAttribute(new Attribute("unitCode", unitCode));
		//cbcLatestMeterQuantity.setAttribute(new Attribute("schemeID", "UN/ECE rec 20"));
		cbcLatestMeterQuantity.setText("" + latestMeterQuantity);
		cacMeterReading.addContent(cbcLatestMeterQuantity);
		
		cacMeterReading.addContent(cbcPreviousMeterReadingMethodCode);
		cbcPreviousMeterReadingMethodCode.setText(previousMeterReadingMethodCode);
		
		cacMeterReading.addContent(cbcLatestMeterReadingMethodCode);
		cbcLatestMeterReadingMethodCode.setText(latestMeterReadingMethodCode);
		
		cbcDeliveredQuantity.setAttribute(new Attribute("unitCode", unitCode));
		//cbcDeliveredQuantity.setAttribute(new Attribute("schemeID", "UN/ECE rec 20"));
		cbcDeliveredQuantity.setText("" + deliveredQuantity);
		cacMeterReading.addContent(cbcDeliveredQuantity);
	}
	
	/**
	 * Aggiunge il tag <i>cac:MeterProperty</i>
	 *
	 * @param cacUtilityMeter (Element) UtilityMeter del doc XML a cui aggiungere il tag 
	 * @param nameCode (String) nome
	 * @param value (String) valore (dato di tipo testuale)
	 * @param valueQuantity (double) valore (dato di tipo numerico)
	 * @param unitCode (String) unita' di misura riferita al valore
	 * 
	 * @return (void)
	 */
	public static void setUSMeterProperty(Element cacUtilityMeter, String nameCode, String value, double valueQuantity, String unitCode) {
		Element cacMeterProperty = new Element("MeterProperty").setNamespace(cac);
		Element cbcNameCode = new Element("NameCode").setNamespace(cbc);
		Element cbcValue = new Element("Value").setNamespace(cbc);
		Element cbcValueQuantity = new Element("ValueQuantity").setNamespace(cbc);

		cacUtilityMeter.addContent(cacMeterProperty);
		cacMeterProperty.addContent(cbcNameCode);
		cbcNameCode.setText(nameCode);
		
		cacMeterProperty.addContent(cbcValue);
		cbcValue.setText(value);
		
		cbcValueQuantity.setAttribute(new Attribute("unitCode", unitCode.toUpperCase()));
		//cbcValueQuantity.setAttribute(new Attribute("schemeID", "UN/ECE rec 20"));
		cbcValueQuantity.setText("" + valueQuantity);
		cacMeterProperty.addContent(cbcValueQuantity);
	}
	
	/**
	 * Aggiunge il tag <i>cac:SupplierConsumption</i>
	 *
	 * @param subscriberConsumption (Element) subscriberConsumption del doc XML a cui aggiungere il tag 
	 * @param name (String) denominazione
	 * @param streetName (String) via
	 * @param buildingNumber (String) numero civico
	 * @param cityName (String) localita'
	 * @param postalZone (String) cap
	 * @param countrySubentity (String) provincia
	 * @param countryId (String) id della nazione (ad esempio IT)
	 * @param countryName (String) nome della nazione
	 * @param companyId (String) numero partita IVA
	 * @param companySchemeId (String) schemad di identificazione di companyId
	 * @param id (String)
	 * @param idSchemeId (String)
	 * 
	 * @return (Element) cacSupplierConsumption 
	 */
	public static Element setUSSupplierConsumption(Element subscriberConsumption, String name, String streetName, String buildingNumber, 
			String cityName, String postalZone, String countrySubentity, String countryId, String countryName,
			String companyId) {
		Element cacSupplierConsumption = new Element("SupplierConsumption").setNamespace(cac);
		Element cacUtilitySupplierParty = new Element("UtilitySupplierParty").setNamespace(cac);
		Element cacPartyName = new Element("PartyName").setNamespace(cac);
		Element cbcName = new Element("Name").setNamespace(cbc);

		subscriberConsumption.addContent(cacSupplierConsumption);
		cacSupplierConsumption.addContent(cacUtilitySupplierParty);
		cacUtilitySupplierParty.addContent(cacPartyName);
		cacPartyName.addContent(cbcName);
		cbcName.setText(name);
		
		Element cacPostalAddress = new Element("PostalAddress").setNamespace(cac);
		
		setIndirizzo(cacUtilitySupplierParty, cacPostalAddress, streetName, buildingNumber, cityName, postalZone, countrySubentity, countryId, countryName);
		
		Element cacPartyTaxScheme = new Element("PartyTaxScheme").setNamespace(cac);
		Element cbcCompanyId = new Element("CompanyID").setNamespace(cbc);
		
		cacUtilitySupplierParty.addContent(cacPartyTaxScheme);
		cbcCompanyId.setAttribute(new Attribute("schemeID", "IT:VAT"));
		cbcCompanyId.setText(companyId);
		cacPartyTaxScheme.addContent(cbcCompanyId);
		setTaxScheme(cacPartyTaxScheme);
		
		return cacSupplierConsumption;
	}
	
	/**
	 * Aggiunge il tag <i>cac:Consumption</i>
	 *
	 * @param supplierConsumption (Element) supplierConsumption del doc XML a cui aggiungere il tag 
	 * @param startDate (String) inizio periodo di calcolo
	 * @param endDate (String) fine periodo di calcolo
	 * @param description (String) operazione
	 * 
	 * @return (Element) cacConsumption 
	 */
	public static Element setUSConsumption(Element supplierConsumption, String startDate, String endDate, String description) {
		Element cacConsumption = new Element("Consumption").setNamespace(cac);
		Element cacMainPeriod = new Element("MainPeriod").setNamespace(cac);
		Element cbcStartDate = new Element("StartDate").setNamespace(cbc);
		Element cbcEndDate = new Element("EndDate").setNamespace(cbc);
		Element cbcDescription = new Element("Description").setNamespace(cbc);

		supplierConsumption.addContent(cacConsumption);
		cacConsumption.addContent(cacMainPeriod);
		if((startDate != null) && (!startDate.equals(""))){
			cacMainPeriod.addContent(cbcStartDate);
			cbcStartDate.setText(formatDateString(startDate));
		}
		if((endDate != null) && (!endDate.equals(""))){
			cacMainPeriod.addContent(cbcEndDate);
			cbcEndDate.setText(formatDateString(endDate));
		}
		if((description != null) && (!description.equals(""))){
			cacMainPeriod.addContent(cbcDescription);
			cbcDescription.setText(description);
		}
		
		return cacConsumption;
	}
	
	/**
	 * Aggiunge il tag <i>cac:AllowanceCharge</i>
	 *
	 * @param consumption (Element) consumption del doc XML a cui aggiungere il tag 
	 * @param chargeIndicator (String) indica se si tratta di un abbuono o di un costo
	 * @param allowanceChargeReason (String) descrizione
	 * @param amount (double) importo
	 * @param unitCode (String) valuta
	 * @param taxCategoryId (String) 
	 * @param taxCategorySchemeId (String) 
	 * @param percent (double) aliquota IVA
	 * @param taxSchemeId (String) identificativo dello schema di imposta
	 * @param taxSchemeSchemeId (String) schema di imposta
	 * 
	 * @return (void) 
	 */
	public static void setUSAllowanceCharge(Element consumption, String chargeIndicator, String allowanceChargeReason, double amount,
			String unitCode, String taxCategoryId, double percent, double taxableAmount, double taxAmount, boolean subTotal) {
		Element cacAllowanceCharge = new Element("AllowanceCharge").setNamespace(cac);
		Element cbcChargeIndicator = new Element("ChargeIndicator").setNamespace(cbc);
		Element cbcAllowanceChargeReason = new Element("AllowanceChargeReason").setNamespace(cbc);
		Element cbcAmount = new Element("Amount").setNamespace(cbc);

		consumption.addContent(cacAllowanceCharge);
		cacAllowanceCharge.addContent(cbcChargeIndicator);
		cbcChargeIndicator.setText(chargeIndicator);
		cacAllowanceCharge.addContent(cbcAllowanceChargeReason);
		cbcAllowanceChargeReason.setText(allowanceChargeReason);
		cbcAmount.setAttribute(new Attribute("unitCode", unitCode));
		cbcAmount.setText("" + amount);
		cacAllowanceCharge.addContent(cbcAmount);
		
		setRiepilogoIva(cbcAllowanceChargeReason, taxCategoryId, percent, "" + taxableAmount, "" + taxAmount, subTotal);
		
//		Element cacTaxCategory = new Element("TaxCategory").setNamespace(cac);
//		Element cbcTaxCategoryId = new Element("ID").setNamespace(cbc);
//		Element cbcPercent = new Element("Percent").setNamespace(cbc);
//		
//		cacAllowanceCharge.addContent(cacTaxCategory);
//		cbcTaxCategoryId.setAttribute(new Attribute("schemeID", taxCategorySchemeId));
//		cbcTaxCategoryId.setText(taxCategoryId);
//		cacTaxCategory.addContent(cbcTaxCategoryId);
//		cacTaxCategory.addContent(cbcPercent);
//		cbcPercent.setText("" + percent);
//		
//		Element cacTaxScheme = new Element("TaxScheme").setNamespace(cac);
//		Element cbcTaxSchemeId = new Element("ID").setNamespace(cbc);
//		
//		cacTaxCategory.addContent(cacTaxScheme);
//		cbcTaxSchemeId.setAttribute(new Attribute("schemeID", taxSchemeSchemeId));
//		cbcTaxSchemeId.setText(taxSchemeId);
//		cacTaxScheme.addContent(cbcTaxSchemeId);
	}
	
	/**
	 * Aggiunge il tag <i>cac:TaxTotal</i>
	 *
	 * @param consumption (Element) consumption del doc XML a cui aggiungere il tag 
	 * @param taxAmount (double) totale imposte
	 * 
	 * @return (Element) cacTaxTotal 
	 */
	public static Element setUSTaxTotal(Element consumption, double taxAmount) {
		Element cacTaxTotal = new Element("TaxTotal").setNamespace(cac);
		Element cbcTaxAmount = new Element("TaxAmount").setNamespace(cbc);

		consumption.addContent(cacTaxTotal);
		cacTaxTotal.addContent(cbcTaxAmount);
		cbcTaxAmount.setText("" + taxAmount);
		
		return cacTaxTotal;
	}
	
	/**
	 * Aggiunge il tag <i>cac:TaxSubtotal</i>
	 *
	 * @param taxTotal (Element) taxTotal del doc XML a cui aggiungere il tag 
	 * @param taxAmount (double) subtotale per una categoria di imposta
	 * @param unitCode (String) valuta
	 * @param name (double) descrizione della categoria
	 * @param perUnitAmount (double) percentuale per unita'
	 * @param tierRange (String) range di applicabilita'
	 * @param id (String) identificativo dello schema di imposta
	 * @param schemeId (double) schema di imposta
	 * 
	 * @return (void) 
	 */
	public static void setUSTaxSubtotal(Element taxTotal, double taxAmount, String unitCode, String name,
			double perUnitAmount, String tierRange, String id, String schemeId) {
		Element cacTaxSubtotal = new Element("TaxSubtotal").setNamespace(cac);
		Element cbcTaxAmount = new Element("TaxAmount").setNamespace(cbc);

		taxTotal.addContent(cacTaxSubtotal);
		cbcTaxAmount.setAttribute(new Attribute("unitCode", unitCode));
		cbcTaxAmount.setText("" + taxAmount);
		taxTotal.addContent(cbcTaxAmount);
		
		Element cacTaxCategory = new Element("TaxCategory").setNamespace(cac);
		Element cbcName = new Element("Name").setNamespace(cbc);
		Element cbcPerUnitAmount = new Element("PerUnitAmount").setNamespace(cbc);
		Element cbcTierRange = new Element("TierRange").setNamespace(cbc);
		
		cacTaxCategory.addContent(cbcName);
		cbcName.setText(name);
		cacTaxCategory.addContent(cbcPerUnitAmount);
		cbcPerUnitAmount.setText("" + perUnitAmount);
		cacTaxCategory.addContent(cbcTierRange);
		cbcTierRange.setText(tierRange);
		
		Element cacTaxScheme = new Element("TaxScheme").setNamespace(cac);
		Element cbcTaxSchemeId = new Element("ID").setNamespace(cbc);
		
		cacTaxCategory.addContent(cacTaxScheme);
		cbcTaxSchemeId.setAttribute(new Attribute("schemeID", schemeId));
		cbcTaxSchemeId.setText(schemeId);
		cacTaxScheme.addContent(cbcTaxSchemeId);
	}
	
	public static Element setUSEnergyWaterSupply(Element consumption) {
		Element cacEnergyWaterSupply = new Element("EnergyWaterSupply").setNamespace(cac);
		consumption.addContent(cacEnergyWaterSupply);
		return cacEnergyWaterSupply;
	}
	
	/**
	 * Aggiunge il tag <i>cac:ConsumptionReport</i>
	 *
	 * @param consumption (Element) consumption del doc XML a cui aggiungere il tag 
	 * @param id (String) identificativo
	 * @param consumptionType (String) tipologia corrispettivo
	 * @param description (String) descrizione
	 * @param totalConsumedQuantity (double) quantita' rilevata
	 * @param unitCode (String) unita' di misura
	 * @param consumersEnergyLevel (String) fascia - scaglione
	 * @param startDate (double) inizio periodo
	 * @param endDate (String) fine periodo
	 * 
	 * @return (Element) cacConsumptionReport 
	 */
	public static Element setUSConsumptionReport(Element energyWaterSupply, String id, String consumptionType, String description,
			double totalConsumedQuantity, String unitCode, String consumersEnergyLevel, String startDate, String endDate) {
		Element cacConsumptionReport = new Element("ConsumptionReport").setNamespace(cac);
		Element cbcId = new Element("ID").setNamespace(cbc);
		Element cbcConsumptionType = new Element("ConsumptionType").setNamespace(cbc);
		Element cbcDescription = new Element("Description").setNamespace(cbc);
		Element cbcTotalConsumedQuantity = new Element("TotalConsumedQuantity").setNamespace(cbc);
		Element cbcConsumersEnergyLevel = new Element("ConsumersEnergyLevel").setNamespace(cbc);

		energyWaterSupply.addContent(cacConsumptionReport);
		cacConsumptionReport.addContent(cbcId);
		cbcId.setText(id);
		cacConsumptionReport.addContent(cbcConsumptionType);
		cbcConsumptionType.setText(consumptionType);
		cacConsumptionReport.addContent(cbcDescription);
		cbcDescription.setText(description);
		cbcTotalConsumedQuantity.setAttribute(new Attribute("unitCode", unitCode.toUpperCase()));
		//cbcTotalConsumedQuantity.setAttribute(new Attribute("schemeID", "UN/ECE rec 20"));
		cbcTotalConsumedQuantity.setText("" + totalConsumedQuantity);
		cacConsumptionReport.addContent(cbcTotalConsumedQuantity);
		cacConsumptionReport.addContent(cbcConsumersEnergyLevel);
		cbcConsumersEnergyLevel.setText(consumersEnergyLevel);
		
		Element cacPeriod = new Element("Period").setNamespace(cac);
		Element cbcStartDate = new Element("StartDate").setNamespace(cbc);
		Element cbcEndDate = new Element("EndDate").setNamespace(cbc);
		
		cacConsumptionReport.addContent(cacPeriod);
		if((startDate != null) && (!startDate.equals(""))){
			cacPeriod.addContent(cbcStartDate);
			cbcStartDate.setText(formatDateString(startDate));
		}
		if((endDate != null) && (!endDate.equals(""))){
			cacPeriod.addContent(cbcEndDate);
			cbcEndDate.setText(formatDateString(endDate));
		}
		
		return cacConsumptionReport;
	}
	
	/**
	 * Aggiunge il tag <i>cac:ConsumptionHistory</i>
	 *
	 * @param consumptionReport (Element) consumptionReport del doc XML a cui aggiungere il tag 
	 * @param meterNumber (String) riferimento a contatore
	 * @param quantity (double) quantita' rilevata dal contatore indicato
	 * @param amount (double) importo dovuto per la quantit' rilevata
	 * @param startDate (double) inizio periodo
	 * @param endDate (String) fine periodo
	 * 
	 * @return (void) 
	 */
	public static void setUSConsumptionHistory(Element consumptionReport, String meterNumber, double quantity, 
			double amount, String startDate, String endDate) {
		Element cacConsumptionHistory = new Element("ConsumptionHistory").setNamespace(cac);
		Element cbcMeterNumber = new Element("MeterNumber").setNamespace(cbc);
		Element cbcQuantity = new Element("Quantity").setNamespace(cbc);
		Element cbcAmount = new Element("Amount").setNamespace(cbc);

		consumptionReport.addContent(cacConsumptionHistory);
		cacConsumptionHistory.addContent(cbcMeterNumber);
		cbcMeterNumber.setText(meterNumber);
		cacConsumptionHistory.addContent(cbcQuantity);
		cbcQuantity.setText("" + quantity);
		cacConsumptionHistory.addContent(cbcAmount);
		cbcAmount.setText("" + amount);
		
		Element cacPeriod = new Element("Period").setNamespace(cac);
		Element cbcStartDate = new Element("StartDate").setNamespace(cbc);
		Element cbcEndDate = new Element("EndDate").setNamespace(cbc);
		
		cacConsumptionHistory.addContent(cacPeriod);
		if((startDate != null) && (!startDate.equals(""))){
			cacPeriod.addContent(cbcStartDate);
			cbcStartDate.setText(formatDateString(startDate));
		}
		if((endDate != null) && (!endDate.equals(""))){
			cacPeriod.addContent(cbcEndDate);
			cbcEndDate.setText(formatDateString(endDate));
		}
	}
	
	/**
	 * Aggiunge il tag <i>cac:LegalMonetaryTotal</i>
	 *
	 * @param consumption (Element) consumption del doc XML a cui aggiungere il tag 
	 * @param payableAmount (double) importo totale dovuto
	 * 
	 * @return (void) 
	 */
	public static void setUSLegalMonetaryTotal(Element consumption, double payableAmount) {
		Element cacLegalMonetaryTotal = new Element("LegalMonetaryTotal").setNamespace(cac);
		Element cbcPayableAmount = new Element("PayableAmount").setNamespace(cbc);

		consumption.addContent(cacLegalMonetaryTotal);
		cbcPayableAmount.setAttribute(new Attribute("currencyID", "EUR"));
		cbcPayableAmount.setText("" + payableAmount);
		cacLegalMonetaryTotal.addContent(cbcPayableAmount);
	}
	
	/**
	 * Aggiunge il tag <i>cac:Contract</i>
	 *
	 * @param supplierConsumption (Element) supplierConsumption del doc XML a cui aggiungere il tag 
	 * @param id (String) codice contratto/convenzione
	 * @param contractType (String) fine periodo di calcolo
	 * @param note (String) contesto geografico e ambito
	 * @param description (String) caratteristiche tecniche
	 * 
	 * @return (void) 
	 */
	public static void setUSContract(Element supplierConsumption, String id, String contractType, String note, String description) {
		Element cacContract = new Element("Contract").setNamespace(cac);
		Element cacId = new Element("ID").setNamespace(cbc);
		Element cbcContractType = new Element("ContractType").setNamespace(cbc);
		Element cbcNote = new Element("Note").setNamespace(cbc);
		Element cbcDescription = new Element("Description").setNamespace(cbc);

		supplierConsumption.addContent(cacContract);
		cacContract.addContent(cacId);
		cacId.setText(id);
		cacContract.addContent(cbcContractType);
		cbcContractType.setText(contractType);
		
		cacContract.addContent(cbcNote);
		cbcNote.setText(note);

		if(description != null && !description.equals("")) {
			cacContract.addContent(cbcDescription);
			cbcDescription.setText(description);
		}
	}
	
	/**
	 * Aggiunge il tag <i>cac:ConsumptionLine</i>
	 *
	 * @param supplierConsumption (Element) supplierConsumption del doc XML a cui aggiungere il tag 
	 * @param id (String)
	 * @param parentDocumentLine (String) riferimento ad una linea di fattura
	 * @param invoicedQuantity (double) quantita' totale per la tipologia
	 * @param invoicedQuantityUnitCode (String) unita' di misura
	 * @param lineExtensionAmount (double) importo totale
	 * @param extensionAmountCurrencyID (String) valuta importo totale
	 * @param startDate (String) data inizio
	 * @param endDate (String) data fine
	 * @param descriptionCode (String) tipologia di periodo
	 * @param taxAmount (double) importo IVA
	 * @param taxAmountCurrencyID (String) valuta importo IVA
	 * @param utilityItemId (String)
	 * @param consumptionTypeCode (String) tipologia corrispettivo
	 * @param utilityItemDescription (String)
	 * @param taxCategoryId (String) categoria di imposta
	 * @param percent (double) IVA
	 * @param priceAmount (double) importo unitario
	 * @param currencyId (String) valuta importo unitario
	 * @param baseQuantity (double) quantita' alla quale si applica l'importo unitario
	 * 
	 * @return (void) 
	 */
	public static void setUSConsumptionLine(Element supplierConsumption, String id, String parentDocumentLine, double invoicedQuantity, 
			String invoicedQuantityUnitCode, double lineExtensionAmount, String extensionAmountCurrencyID, String startDate, String endDate,
			String descriptionCode, double taxAmount, String taxAmountCurrencyID, String utilityItemId, String consumptionTypeCode,
			String taxCategoryId, double percent, double priceAmount, String priceAmountCurrencyId, double baseQuantity) {
		
		Element cacConsumptionLine = new Element("ConsumptionLine").setNamespace(cac);
		Element cbcId = new Element("ID").setNamespace(cbc);
		Element cbcParentDocumentLine = new Element("ParentDocumentLineReferenceID").setNamespace(cbc);
		Element cbcInvoicedQuantity = new Element("InvoicedQuantity").setNamespace(cbc);
		Element cbcLineExtensionAmount = new Element("LineExtensionAmount").setNamespace(cbc);

		supplierConsumption.addContent(cacConsumptionLine);
		cacConsumptionLine.addContent(cbcId);
		cbcId.setText(id);
		cacConsumptionLine.addContent(cbcParentDocumentLine);
		cbcParentDocumentLine.setText(parentDocumentLine);
		cbcInvoicedQuantity.setAttribute(new Attribute("unitCode", invoicedQuantityUnitCode.toUpperCase()));
		//cbcInvoicedQuantity.setAttribute(new Attribute("schemeID", "UN/ECE rec 20"));
		cbcInvoicedQuantity.setText("" + invoicedQuantity);
		cacConsumptionLine.addContent(cbcInvoicedQuantity);
		cbcLineExtensionAmount.setAttribute(new Attribute("currencyID", extensionAmountCurrencyID));
		cbcLineExtensionAmount.setText("" + lineExtensionAmount);
		cacConsumptionLine.addContent(cbcLineExtensionAmount);
		
		Element cacPeriod = new Element("Period").setNamespace(cac);
		Element cbcStartDate = new Element("StartDate").setNamespace(cbc);
		Element cbcEndDate = new Element("EndDate").setNamespace(cbc);
		Element cbcDescription = new Element("DescriptionCode").setNamespace(cbc);
		
		cacConsumptionLine.addContent(cacPeriod);
		if((startDate != null) && (!startDate.equals(""))){
			cacPeriod.addContent(cbcStartDate);
			cbcStartDate.setText(formatDateString(startDate));
		}
		if((endDate != null) && (!endDate.equals(""))){
			cacPeriod.addContent(cbcEndDate);
			cbcEndDate.setText(formatDateString(endDate));
		}
		if((descriptionCode != null) && (!descriptionCode.equals(""))){
			cacPeriod.addContent(cbcDescription);
			cbcDescription.setText(descriptionCode);
		}
		
		Element cacTaxTotal = new Element("TaxTotal").setNamespace(cac);
		Element cbcTaxAmount = new Element("TaxAmount").setNamespace(cbc);

		cacConsumptionLine.addContent(cacTaxTotal);
		cbcTaxAmount.setAttribute(new Attribute("currencyID", taxAmountCurrencyID));
		cbcTaxAmount.setText("" + taxAmount);
		cacTaxTotal.addContent(cbcTaxAmount);
		
		Element cacUtilityItem = new Element("UtilityItem").setNamespace(cac);
		Element cbcUtilityItemId = new Element("ID").setNamespace(cbc);
		Element cbcConsumptionTypeCode = new Element("ConsumptionTypeCode").setNamespace(cbc);
		
		cacConsumptionLine.addContent(cacUtilityItem);
		cacUtilityItem.addContent(cbcUtilityItemId);
		cbcUtilityItemId.setText(utilityItemId);
		cacUtilityItem.addContent(cbcConsumptionTypeCode);
		cbcConsumptionTypeCode.setText(consumptionTypeCode);
		
		Element cacTaxCategory = new Element("TaxCategory").setNamespace(cac);
		cacUtilityItem.addContent(cacTaxCategory);
		setTaxCategory(cacTaxCategory, taxCategoryId, percent, null);
		
		Element cacPrice = new Element("Price").setNamespace(cac);
		Element cbcPriceAmount = new Element("PriceAmount").setNamespace(cbc);
		Element cbcBaseQuantity = new Element("BaseQuantity").setNamespace(cbc);
		
		cacConsumptionLine.addContent(cacPrice);
		cbcPriceAmount.setAttribute(new Attribute("currencyID", priceAmountCurrencyId));
		cbcPriceAmount.setText(new BigDecimal(Double.toString(priceAmount)).toPlainString());
		cacPrice.addContent(cbcPriceAmount);
		cacPrice.addContent(cbcBaseQuantity);
		cbcBaseQuantity.setText("" + baseQuantity);
		
	}
	
	public static String formatDateString(String strDate) {
		SimpleDateFormat formatterDate;
		SimpleDateFormat formatterString;
		Date date;  
		try {
		     formatterDate = new SimpleDateFormat("yyyyMMdd");
		     date = (Date)formatterDate.parse(strDate);
		     
             formatterString = new SimpleDateFormat("yyyy-MM-dd");            
		     String s = formatterString.format(date);
		     return s;
		 }
		 catch (ParseException e){
			 return null;
		 }
	}   
}
