/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.invoice;

import it.fe.core.par.common.PostalAddress;

import java.io.Serializable;
import java.util.List;

import javax.persistence.AttributeOverride;
import javax.persistence.AttributeOverrides;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;

/**
 * La classe <b><i>Client</i></b> contiene le informazioni, necessare in fattura, relative al cliente.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 * 
 */
@Entity
@Table(name="Clients")
public class Client implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	/**
	 * Identificativo.
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long clientId;
	
	
	/**
	 * Ragione sociale.
	 */
	@Column(nullable=false)
	private String clientName;
	
	
	/**
	 * Il VAT ID Europeo è un codice alfanumerico unico rilasciato da ogni nazione ai propri cittadini ed organizzazioni.
	 * In Italia è la Partita Iva.
	 */
	@Column(nullable=false)
	private String vatId;
	
	/**
	 * Codice fiscale.
	 */
	@Column
	private String italianFiscalCode;
	

	/**
	 * Indirizzo dell'ente a cui sono intestate le fatture.
	 * 
	 */
	@Embedded
	private PostalAddress postalAddress;
	
	
	/**
	 * Indirizzo della sede legale dell'ente.
	 * 
	 */
	@Embedded
	@AttributeOverrides( {
		@AttributeOverride(name = "postBox",
		column = @Column(name="registrationAddress_postBox") ),
		@AttributeOverride(name = "streetName",
		column = @Column(name="registrationAddress_streetName") ),
		@AttributeOverride(name = "floor",
		column = @Column(name="registrationAddress_floor") ),
		@AttributeOverride(name = "room",
		column = @Column(name="registrationAddress_room") ),
		@AttributeOverride(name = "additionalStreetName",
		column = @Column(name="registrationAddress_additionalStreetName") ),
		@AttributeOverride(name = "buildingNumber",
		column = @Column(name="registrationAddress_buildingNumber") ),
		@AttributeOverride(name = "department",
		column = @Column(name="registrationAddress_department") ),
		@AttributeOverride(name = "cityName",
		column = @Column(name="registrationAddress_cityName") ),
		@AttributeOverride(name = "postalZone",
		column = @Column(name="registrationAddress_postalZone") ),
		@AttributeOverride(name = "countrySubentityCode",
		column = @Column(name="registrationAddress_countrySubentityCode") ),
		@AttributeOverride(name = "country",
		column = @Column(name="registrationAddress_country")),
		@AttributeOverride(name = "countryName",
		column = @Column(name="registrationAddress_countryName"))
		})
	private PostalAddress registrationAddress;
	
	
	/**
	 * Lista delle fatture intestate al cliente.
	 */
	@OneToMany(	mappedBy="client", 
			fetch=FetchType.LAZY)
	private List<Invoice> invoices;


	
	//--------------- Ctors --------------------------
	
	public Client() {
		super();
	}


	/**
	 * @param clientName
	 * @param vatID
	 * @param postalAddress
	 */
	public Client(String clientName, String vatID,
			PostalAddress postalAddress) {
		super();
		this.clientName = clientName;
		this.vatId = vatID;
		this.postalAddress = postalAddress;
	}

	
	
	//--------------- Getters/Setters --------------------------

	/**
	 * @return the clientId
	 */
	public Long getClientId() {
		return clientId;
	}


	/**
	 * @param clientId the clientId to set
	 */
	public void setClientId(Long clientId) {
		this.clientId = clientId;
	}


	/**
	 * @return the clientName
	 */
	public String getClientName() {
		return clientName;
	}


	/**
	 * @param clientName the clientName to set
	 */
	public void setClientName(String clientName) {
		this.clientName = clientName;
	}


	/**
	 * @return the vatId
	 */
	public String getVatId() {
		return vatId;
	}


	/**
	 * @param vatId the vatId to set
	 */
	public void setVatId(String vatId) {
		this.vatId = vatId;
	}


	/**
	 * @param italianFiscalCode the italianFiscalCode to set
	 */
	public void setItalianFiscalCode(String italianFiscalCode) {
		this.italianFiscalCode = italianFiscalCode;
	}


	/**
	 * @return the italianFiscalCode
	 */
	public String getItalianFiscalCode() {
		return italianFiscalCode;
	}


	/**
	 * @return the postalAddress
	 */
	public PostalAddress getPostalAddress() {
		return postalAddress;
	}


	/**
	 * @param postalAddress the postalAddress to set
	 */
	public void setPostalAddress(PostalAddress postalAddress) {
		this.postalAddress = postalAddress;
	}


	/**
	 * @param registrationAddress the registrationAddress to set
	 */
	public void setRegistrationAddress(PostalAddress registrationAddress) {
		this.registrationAddress = registrationAddress;
	}


	/**
	 * @return the registrationAddress
	 */
	public PostalAddress getRegistrationAddress() {
		return registrationAddress;
	}


	/**
	 * @return the invoices
	 */
	public List<Invoice> getInvoices() {
		return invoices;
	}


	/**
	 * @param invoices the invoices to set
	 */
	public void setInvoices(List<Invoice> invoices) {
		this.invoices = invoices;
	}
	
}
