/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 * <br /><br />
* 
 *
 */
package it.fe.core.par.invoice;

import it.fe.core.par.common.FEFile;
import it.fe.core.par.utility.MeterSummary;
import it.fe.core.par.utility.Reading;
import it.fe.core.par.utility.UtilitySupplier;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;


/**
 * La classe <b><i>Invoice</i></b> contiene i dati obbligatori secondo la normativa italiana per definire una fattura come tale.
 * Vegono, inoltre, mantenuti i riferimenti alle categorie di dati associati:
 * <ul>
 * 	<li>Cliente</li>
 * 	<li>Fornitore</li>
 * 	<li>RigaFattura</li>
 * 	<li>DettaglioIva</li>
 * 	<li>PeriodoConsumo</li>
 * 	<li>DatiPagamento</li>
 * 	<li>DocumentiPregressi</li>
 * 	<li>DatiAggiuntivi</li>
 * </ul>
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 * 
 * @author chiara.pezzi@epocaricerca.it, fulvio.dimarco@epocaricerca.it
 *
 */
@Entity
@Table(name="Invoices")
public class Invoice implements Serializable {
	
	private static final long serialVersionUID = 1L;

	public static final class DocumentType {
		public static final String Invoice ="Invoice";
		public static final String CreditNote ="CreditNote";
		public static final String DebitNote ="DebitNote";
		
	};

	/**
	 * Tipologia di fattura: fattura standard, nota di credito, nota di debito
	 */
	private String documentType;
	
	/**
	 * Errore durante l'elaborazione
	 */
	@Column(columnDefinition="boolean default '0'")
	private boolean error;
	
	
	/**
	 * Messaggio di errore
	 */
	private String errorMessage;
	
	
	/**
	 * Numero progressivo assegnato alla fattura dal cliente (es. protocollo)
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long invoiceId;
	
	
	/**
	 * Data di emissione della fattura, riportata sulla fattura stessa.
	 */
	@Temporal(TemporalType.DATE)
	@Column(nullable=false)
	private Date issueDate;
	
	
	/**
	 * Numero della fattura assegnato dal fornitore.
	 */
	@Column(nullable=false)
	private String invoiceNumber;
	
	
	/**
	 * Riferimento ai dati relativi al cliente (ragione sociale, indirizzo, partita IVA).
	 * Si presume una fattura possa avere un solo cliente e che un cliente possa avere più fatture.
	 */
	@ManyToOne
	@JoinColumn(name="clientId", referencedColumnName="clientId", nullable=false)
	private Client client;
	
	
	/**
	 * Riferimento ai dati relativi al fornitore (ragione sociale, indirizzo, partita IVA o codice fiscale).
	 * Si presume una fattura possa avere un solo fornitore e che un fornitore possa emettere più fatture.
	 */
	@ManyToOne
	@JoinColumn(name="supplierId", referencedColumnName="supplierId", nullable=false)
	private Supplier supplier;
	
	
	/**
	 * Totale complessivo della fattura.
	 * 
	 */
	@Column(nullable=false)
	private Double allInclusiveTotal;
	
	/**
	 * Valuta della fattura.
	 * 
	 */
	private String allInclusiveTotalCurrency;
	
	
	/**
	 * Riferimenti a sconti, premi, abbuoni.
	 */
	@OneToMany( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="invoiceId", referencedColumnName="invoiceId")
	private List<AllowanceCharge> allowanceCharges;
	
	
	/**
	 * La data della fattura, usata per indicare il momento in cui l'imposta sarà applicabile.
	 * 
	 */
	@Temporal(TemporalType.DATE)
	private Date taxPointDate;
	
	
	/**
	 * Riferimento ai dati relativi al dettaglio IVA (base imponibile, aliquota, arrotondamento...).
	 * E' possibile che il dettaglio iva sia composto da più parti.
	 */
	@OneToMany(	fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="invoiceId", referencedColumnName="invoiceId")
	private List<TaxDetail> taxDetails;

	
	/**
	 * Riferimento ai dati relativi al pagamento (scadenza, modalità di pagamento...).
	 * Si presume che ogni fattura abbia un dettaglio di pagamento unico.
	 */
	@OneToOne( fetch=FetchType.EAGER, cascade=CascadeType.ALL )
	@JoinColumn(name="paymentDetailId", referencedColumnName="paymentDetailId")
	private PaymentDetail paymentDetail;

	
	/**
	 * Riferimento ai dati relativi al periodo di consumo.
	 */
	@OneToOne( fetch=FetchType.EAGER, 
			cascade=CascadeType.ALL )
	@JoinColumn(name="invoicePeriodId", referencedColumnName="invoicePeriodId")
	private InvoicePeriod invoicePeriod;

	
	/**
	 * Il Centro di Costo contabile dell'acquirente applicato all'intera Fattura, espresso come testo.
	 * 
	 */
	private String accountingCost;
	
	/**
	 * Riferimento ai dati relativi ai documenti che precedono la fattura (ordine, contratto...).
	 * E' possibile, ad esempio, che una fattura faccia riferimento a più ordini.
	 */
	@ManyToMany( fetch=FetchType.LAZY )
	private List<PreviousDocument> previousDocuments;
	

	/**
	 * Lista di righe della fattura
	 */
	@OneToMany( mappedBy="invoice", fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	private List<InvoiceLine> invoiceLines;
	
	/**
	 * Note.
	 */
	@OneToMany( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="invoiceId", referencedColumnName="invoiceId")
	private List<InvoiceNote> noteList;
	
	
	/*** UTILITY INVOICES ***/
	
	/**
	 * Letture
	 */
	@OneToMany( fetch=FetchType.LAZY, cascade=CascadeType.PERSIST )
	@JoinColumn(name="invoiceId", referencedColumnName="invoiceId")
	private List<Reading> readingList;
	
	/**
	 * Riassunto contatore
	 */
	@OneToMany( fetch=FetchType.LAZY, cascade=CascadeType.PERSIST )
	@JoinColumn(name="invoiceId", referencedColumnName="invoiceId")
	private List<MeterSummary> meterSummary;
	
	
	/*** INVOICE MANAGEMENT ***/
	
	/**
	 * Stato della fattura.
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="invoiceStateId", referencedColumnName="invoiceStateId", nullable=false)
	private InvoiceState invoiceState;
	
	/**
	 * Data di ricezione della fattura
	 */
	@Temporal(TemporalType.DATE)
	@Column(nullable=false)
	private Date receiptDate;
		
	
	/**
	 * Riferimento alla fattura in formato XML
	 */
	@OneToOne( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="fileId", referencedColumnName="fileId")
	private FEFile xmlFile;
	
	
	/**
	 * Riferimento alla ProcessInstance JBPM relativa alla fattura
	 */
	@Column(nullable=true)
	private String processInstanceId;

	
	/**
	 * Riferimento ai dati relativi al dettaglio IVA (base imponibile, aliquota, arrotondamento...).
	 * E' possibile che il dettaglio iva sia composto da più parti.
	 */
	@ManyToMany( fetch=FetchType.LAZY )
	@JoinTable(joinColumns = @JoinColumn(name="invoiceId", referencedColumnName="invoiceId"))
	private List<OriginalDocument> originalDocuments;
	
	
	/**
	 * Riferimento ad eventuali dati aggiuntivi
	 */
	@ManyToMany(cascade=CascadeType.ALL, fetch=FetchType.LAZY)
	@JoinTable(name = "Invoices_AdditionalData", joinColumns = @JoinColumn(name = "invoiceId"), inverseJoinColumns = @JoinColumn(name = "additionalDataId"))
	private List<AdditionalData> additionalData;
	
	/**
	 * Riferimento al gestore
	 */
	@OneToOne( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="utilitySupplierId", referencedColumnName="utilitySupplierId")
	private UtilitySupplier utilitySupplier;
	
	//--------------- Ctors --------------------------
	
	public Invoice() {
		super();
	}

	
	public Invoice(Date issueDate, String invoiceNumber,
						Client client, Supplier supplier, Double allInclusiveTotal, 
						List<TaxDetail> taxDetails, PaymentDetail paymentDetail, 
						InvoicePeriod invoicePeriod, List<PreviousDocument> previousDocuments,
						List<InvoiceLine> invoiceLines) {
		super();
		this.issueDate = issueDate;
		this.invoiceNumber = invoiceNumber;
		this.client = client;
		this.supplier = supplier;
		this.allInclusiveTotal = allInclusiveTotal;
		this.taxDetails = taxDetails;
		this.paymentDetail = paymentDetail;
		this.invoicePeriod = invoicePeriod;
		this.previousDocuments = previousDocuments;
		this.invoiceLines = invoiceLines;
		this.documentType = Invoice.DocumentType.Invoice;
}


	//--------------- Getters/Setters --------------------------
	
	/**
	 * @return the invoiceId
	 */
	public Long getInvoiceId() {
		return invoiceId;
	}


	/**
	 * @param invoiceId the invoiceId to set
	 */
	public void setInvoiceId(long invoiceId) {
		this.invoiceId = invoiceId;
	}


	/**
	 * @return the issueDate
	 */
	public Date getIssueDate() {
		return issueDate;
	}


	/**
	 * @param issueDate the issueDate to set
	 */
	public void setIssueDate(Date issueDate) {
		this.issueDate = issueDate;
	}


	/**
	 * @return the invoiceNumber
	 */
	public String getInvoiceNumber() {
		return invoiceNumber;
	}


	/**
	 * @param invoiceNumber the invoiceNumber to set
	 */
	public void setInvoiceNumber(String invoiceNumber) {
		this.invoiceNumber = invoiceNumber;
	}


	/**
	 * @return the client
	 */
	public Client getClient() {
		return client;
	}


	/**
	 * @param client the client to set
	 */
	public void setClient(Client client) {
		this.client = client;
	}


	/**
	 * @return the supplier
	 */
	public Supplier getSupplier() {
		return supplier;
	}


	/**
	 * @param supplier the supplier to set
	 */
	public void setSupplier(Supplier supplier) {
		this.supplier = supplier;
	}


	/**
	 * @return the allInclusiveTotal
	 */
	public Double getAllInclusiveTotal() {
		return allInclusiveTotal;
	}


	/**
	 * @param allInclusiveTotal the allInclusiveTotal to set
	 */
	public void setAllInclusiveTotal(Double allInclusiveTotal) {
		this.allInclusiveTotal = allInclusiveTotal;
	}


	/**
	 * @param allInclusiveTotalCurrency the allInclusiveTotalCurrency to set
	 */
	public void setAllInclusiveTotalCurrency(String allInclusiveTotalCurrency) {
		this.allInclusiveTotalCurrency = allInclusiveTotalCurrency;
	}


	/**
	 * @return the allInclusiveTotalCurrency
	 */
	public String getAllInclusiveTotalCurrency() {
		return allInclusiveTotalCurrency;
	}


	/**
	 * @param allowanceCharges the allowanceCharges to set
	 */
	public void setAllowanceCharges(List<AllowanceCharge> allowanceCharges) {
		this.allowanceCharges = allowanceCharges;
	}


	/**
	 * @return the allowanceCharges
	 */
	public List<AllowanceCharge> getAllowanceCharges() {
		return allowanceCharges;
	}


	/**
	 * @param taxPointDate the taxPointDate to set
	 */
	public void setTaxPointDate(Date taxPointDate) {
		this.taxPointDate = taxPointDate;
	}


	/**
	 * @return the taxPointDate
	 */
	public Date getTaxPointDate() {
		return taxPointDate;
	}


	/**
	 * @return the taxDetails
	 */
	public List<TaxDetail> getTaxDetails() {
		if(taxDetails == null) {
			taxDetails = new ArrayList<TaxDetail>();
		}
		
		return taxDetails;
	}


	/**
	 * @param taxDetails the taxDetails to set
	 * 
	 * <br /><br />
	 * N.B. Per settare il dettaglio iva in fase di inserimento della fattura su db
	 * è necessario usare i metodi: <b><i>addTaxDetail</i></b> e <b><i>localAddTaxDetail</i></b>
	 * <br /><br />
	 * 
	 * @see #addTaxDetail(TaxDetail)
	 * @see #localAddTaxDetail(TaxDetail)
	 */
	public void setTaxDetails(List<TaxDetail> taxDetails) {
		this.taxDetails = taxDetails;
	}


	/**
	 * @return the paymentDetail
	 */
	public PaymentDetail getPaymentDetail() {
		return paymentDetail;
	}


	/**
	 * @param paymentDetail the paymentDetail to set
	 */
	public void setPaymentDetail(PaymentDetail paymentDetail) {
		this.paymentDetail = paymentDetail;
	}


	/**
	 * @return the invoicePeriod
	 */
	public InvoicePeriod getInvoicePeriod() {
		return invoicePeriod;
	}


	/**
	 * @param invoicePeriod the invoicePeriod to set
	 */
	public void setInvoicePeriod(InvoicePeriod invoicePeriod) {
		this.invoicePeriod = invoicePeriod;
	}

	
	/**
	 * @param accountingCost the accountingCost to set
	 */
	public void setAccountingCost(String accountingCost) {
		this.accountingCost = accountingCost;
	}


	/**
	 * @return the accountingCost
	 */
	public String getAccountingCost() {
		return accountingCost;
	}


	/**
	 * @return the previousDocument
	 */
	public List<PreviousDocument> getPreviousDocuments() {
		if(previousDocuments == null) {
			previousDocuments = new ArrayList<PreviousDocument>();
		}
		
		return previousDocuments;
	}

	
	/**
	 * @param previousDocument the previousDocument to set
	 */
	public void setPreviousDocuments(List<PreviousDocument> previousDocuments) {
		this.previousDocuments = previousDocuments;
	}


	/**
	 * @return the invoiceLines
	 */
	public List<InvoiceLine> getInvoiceLines() {
		if(invoiceLines == null) {
			invoiceLines = new ArrayList<InvoiceLine>();
		}
		
		return invoiceLines;
	}


	/**
	 * @param invoiceLines the invoiceLines to set
	 * 
	 * <br /><br />
	 * N.B. Per settare il dettaglio iva in fase di inserimento della fattura su db
	 * è necessario usare i metodi: <b><i>addInvoiceLine</i></b> e <b><i>localInvoiceLine</i></b>
	 * <br /><br />
	 * 
	 * @see #addTaxDetail(InvoiceLine)
	 * @see #localAddTaxDetail(InvoiceLine)
	 */
	public void setInvoiceLines(List<InvoiceLine> invoiceLines) {
		this.invoiceLines = invoiceLines;
	}


	/**
	 * @param noteList the noteList to set
	 */
	public void setNoteList(List<InvoiceNote> noteList) {
		this.noteList = noteList;
	}


	/**
	 * @return the noteList
	 */
	public List<InvoiceNote> getNoteList() {
		return noteList;
	}


	/**
	 * @param reading the reading to set
	 */
	public void setReadingList(List<Reading> readingList) {
		this.readingList = readingList;
	}


	/**
	 * @return the reading
	 */
	public List<Reading> getReadingList() {
		return readingList;
	}


	/**
	 * @param meterSummary the meterSummary to set
	 */
	public void setMeterSummary(List<MeterSummary> meterSummary) {
		this.meterSummary = meterSummary;
	}


	/**
	 * @return the meterSummary
	 */
	public List<MeterSummary> getMeterSummary() {
		return meterSummary;
	}


	/**
	 * @return the invoiceState
	 */
	public InvoiceState getInvoiceState() {
		return invoiceState;
	}


	/**
	 * @param invoiceState the invoiceState to set
	 */
	public void setInvoiceState(InvoiceState invoiceState) {
		this.invoiceState = invoiceState;
	}

	
	/**
	 * @return the receiptDate
	 */
	public Date getReceiptDate() {
		return receiptDate;
	}


	/**
	 * @param receiptDate the receiptDate to set
	 */
	public void setReceiptDate(Date receiptDate) {
		this.receiptDate = receiptDate;
	}

	

	

	//----- Helper methods ---------------------------------------

	/**
	 * Imposta la relazione tra InvoiceLine e Invoice
	 * 
	 * @param invoiceLine
	 */
	public void addInvoiceLine(InvoiceLine invoiceLine) {
		invoiceLine.setInvoice(this);
	}
	
	public void removeInvoiceLine(InvoiceLine invoiceLine) {
		invoiceLine.setInvoice(null);
	}
	
	
	/**
	 * Aggiunge una InvoiceLine alla lista invoiceLine della fattura
	 * 
	 * @param taxDetail
	 */
	public void localAddInvoiceLine(InvoiceLine invoiceLine) {
		if (invoiceLines == null)
			invoiceLines = new ArrayList<InvoiceLine>();
		invoiceLines.add(invoiceLine);
	}
	
	public void localRemoveInvoiceLine(InvoiceLine taxDetail) {
		if (invoiceLines == null)
			invoiceLines = new ArrayList<InvoiceLine>();
		invoiceLines.remove(taxDetail);
	}


	public void setProcessInstanceId(String processInstanceId) {
		this.processInstanceId = processInstanceId;
	}


	public String getProcessInstanceId() {
		return processInstanceId;
	}


	/**
	 * @param xmlFile the xmlFile to set
	 */
	public void setXmlFile(FEFile xmlFile) {
		this.xmlFile = xmlFile;
	}


	/**
	 * @return the xmlFile
	 */
	public FEFile getXmlFile() {
		return xmlFile;
	}


	/**
	 * @param originalDocuments the originalDocuments to set
	 */
	public void setOriginalDocuments(List<OriginalDocument> originalDocuments) {
		this.originalDocuments = originalDocuments;
	}

	/**
	 * @return the originalDocuments
	 */
	public List<OriginalDocument> getOriginalDocuments() {
		return originalDocuments;
	}


	public boolean isError() {
		return error;
	}


	public void setError(boolean error) {
		this.error = error;
	}


	public String getErrorMessage() {
		return errorMessage;
	}


	public void setErrorMessage(String errorMessage) {
		this.errorMessage = errorMessage;
	}


	public void setAdditionalData(List<AdditionalData> additionalData) {
		this.additionalData = additionalData;
	}


	public List<AdditionalData> getAdditionalData() {
		return additionalData;
	}


	/**
	 * @return the utilitySupplier
	 */
	public UtilitySupplier getUtilitySupplier() {
		return utilitySupplier;
	}


	/**
	 * @param utilitySupplier the utilitySupplier to set
	 */
	public void setUtilitySupplier(UtilitySupplier utilitySupplier) {
		this.utilitySupplier = utilitySupplier;
	}


	public String getDocumentType() {
		return documentType;
	}


	public void setDocumentType(String documentType) {
		this.documentType = documentType;
	}


	
//	/**
//	 * Logica di reperimento del responsabile valutazione: sulla base del tipo del riferimento e 
//	 * del riferimento stesso, 
//	 * restituisce l'identità del responsabile della validazione.
//	 * 
//	 */
//	@Transient
//	public String getResponsabileValidazione() {
//		if (validatore != null && !validatore.equals("") )
//			return validatore;
//		
//		if (tipoRiferimento.equals(TipoRiferimento.CONTRATTO_ORDINE)){
//			//throw new RuntimeException("getResponsabileValidazione -  CONTRATTO_ORDINE - not yet implemented ");
//			//TODO: accesso al DB alla ricerca del contratto, dal quale ricavare il responsabile
//			return "contratto";
//		}
//		else if (tipoRiferimento.equals(TipoRiferimento.FORNITORE)){
//			//throw new RuntimeException("getResponsabileValidazione -  FORNITORE - not yet implemented ");
//
//			//TODO: accesso al DB alla ricerca del fornitore, dal quale ricavare il responsabile
//			return "fornitore";
//		}
//		else if (tipoRiferimento.equals(TipoRiferimento.RIF_ORGANIZZATIVO)){
//			//TODO: accesso al WS che mappa l'organigramma aziendale
//			//NB:Per ora si suppone che idRiferimento sia = all' id del valutatore incaricato
//			return idRiferimento;
//		}
//		else if ((validatore != null) && !(validatore.equals(""))){
//			return validatore;
//		}
//		else
//			return null;
//	}


}
