/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @author chiara.pezzi@epocaricerca.it
 * 
 *
 *
 */
package it.fe.core.par.invoice;

import it.fe.core.par.common.FEFile;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * La classe <b><i>OriginalDocument</i></b> contiene le informazioni relative ai documenti ricevuti, non ancora processati
 * e riconosciuti come fattura.
 * 
 * <br /><br />
 * 
 * <b>N.B.</b> Sono presenti le annotazioni Hibernate/JPA per la persistenza sul database di FE.
 *
 */
@Entity
@Table(name="OriginalDocuments")
public class OriginalDocument implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	public enum receiveStatusType { OK, ERRORE }

	/**
	 * Identificativo
	 */
	@Id
	@GeneratedValue(strategy=GenerationType.AUTO)
	private Long originalDocumentId;
	
	/**
	 * Identificativo del mezzo di trasmissione tramite cui è stato ricevuto il documento
	 */
	@ManyToOne( fetch=FetchType.EAGER )
	@JoinColumn(name="transmissionTypeId", referencedColumnName="transmissionTypeId", nullable=false)
	private TransmissionType transmissionType;
	
	/**
	 * Data e ora in cui il flusso elettronico è stato ricevuto.
	 */
	@Column(nullable=false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date receiptDateTime;
	
	/**
	 * Documenti ricevuti
	 */
	@OneToMany( fetch=FetchType.LAZY, cascade=CascadeType.ALL )
	@JoinColumn(name="originalDocumentId", referencedColumnName="originalDocumentId")
	private List<FEFile> receiptDocuments;

	
	/**
	 * Provenienza
	 */
	@Column(nullable=false)
	private String sender;
	
	/**
	 * Destinatario
	 */
	@Column
	private String recipient;
	
	/**
	 * Informazioni aggiuntive relative al flusso ricevuto (es. corpo della e-mail)
	 */
	@Lob
	private String additionalInfo;
	
	/**
	 * Stato della ricezione
	 */
	private String receiveStatus;
	
	/**
	 * Messaggio legato all'eccezione sollevata
	 */
	@Lob
	private String exceptionMessage;
	

	/**
	 * @param transmissionType
	 * @param receiptDateTime
	 * @param sender
	 * @param additionalInfo
	 */
	public OriginalDocument(TransmissionType transmissionType,
			Date receiptDateTime, String sender, String recipient, String additionalInfo) {
		super();
		this.transmissionType = transmissionType;
		this.receiptDateTime = receiptDateTime;
		this.sender = sender;
		this.recipient = recipient;
		this.additionalInfo = additionalInfo;
	}

	public OriginalDocument() {
		super();
	}


	/**
	 * @return the originalDocumentId
	 */
	public Long getOriginalDocumentId() {
		return originalDocumentId;
	}

	/**
	 * @param originalDocumentId the originalDocumentId to set
	 */
	public void setOriginalDocumentId(Long originalDocumentId) {
		this.originalDocumentId = originalDocumentId;
	}

	/**
	 * @return the trasmissionType
	 */
	public TransmissionType getTransmissionType() {
		return transmissionType;
	}

	/**
	 * @param transmissionType the trasmissionType to set
	 */
	public void setTransmissionType(TransmissionType transmissionType) {
		this.transmissionType = transmissionType;
	}

	/**
	 * @return the receiptDateTime
	 */
	public Date getReceiptDateTime() {
		return receiptDateTime;
	}

	/**
	 * @param receiptDateTime the receiptDateTime to set
	 */
	public void setReceiptDateTime(Date receiptDateTime) {
		this.receiptDateTime = receiptDateTime;
	}

	/**
	 * @return the documents
	 */
	public List<FEFile> getReceiptDocuments() {
		return receiptDocuments;
	}

	/**
	 * @param documents the documents to set
	 */
	public void setReceiptDocuments(List<FEFile> receiptDocuments) {
		this.receiptDocuments = receiptDocuments;
	}

	/**
	 * @return the sender
	 */
	public String getSender() {
		return sender;
	}

	/**
	 * @param sender the sender to set
	 */
	public void setSender(String sender) {
		this.sender = sender;
	}

	/**
	 * @return the additionalInfo
	 */
	public String getAdditionalInfo() {
		return additionalInfo;
	}

	/**
	 * @param additionalInfo the additionalInfo to set
	 */
	public void setAdditionalInfo(String additionalInfo) {
		this.additionalInfo = additionalInfo;
	}
	
	
	/**
	 * @param receiveStatus the receiveStatus to set
	 */
	public void setReceiveStatus(String receiveStatus) {
		this.receiveStatus = receiveStatus;
	}

	/**
	 * @return the receiveStatus
	 */
	public String getReceiveStatus() {
		return receiveStatus;
	}

	/**
	 * @param exceptionMessage the exceptionMessage to set
	 */
	public void setExceptionMessage(String exceptionMessage) {
		this.exceptionMessage = exceptionMessage;
	}

	/**
	 * @return the exceptionMessage
	 */
	public String getExceptionMessage() {
		return exceptionMessage;
	}

	public String getRecipient() {
		return recipient;
	}

	public void setRecipient(String recipient) {
		this.recipient = recipient;
	}
	
	
}
