/**
 * This file is part of FatturazioneElettronica - Core
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Core is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Core.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.core.process.proxy;

import it.fe.core.common.utils.FEPropertiesUtil;
import it.fe.core.process.exception.JbpmProcessException;

import java.util.Hashtable;

import javax.naming.InitialContext;

import org.apache.log4j.Logger;
import org.jbpm.api.Configuration;
import org.jbpm.api.ExecutionService;
import org.jbpm.api.HistoryService;
import org.jbpm.api.ManagementService;
import org.jbpm.api.ProcessEngine;
import org.jbpm.api.RepositoryService;
import org.jbpm.api.TaskService;

/**
 * @author fulvio di marco
 * 
*
 *
 * 	<p>Component providing an abstraction layer between session beans and jBPM framework,
 *	in order to decouple the application layer from business process management details.
 *	This way, the switch between different BPM engines or between different jBPM versions
 *	is smoother.</p>
 *
 *  <p>A ProcessEngine is thread safe and can be stored in a static member field or even better 
 *  in JNDI or some other central location. One ProcessEngine object can be used by all requests 
 *  and threads in an application.</p>
 *
 */
public class ProcessEngineProxy {
	
	private static Logger logger = Logger.getLogger(ProcessEngineProxy.class); 

	private static ProcessEngine processEngine;
	
	private static String hostPort = FEPropertiesUtil.getJndiHost()+":"+FEPropertiesUtil.getJndiPort();
	
	/**
	 * Returns a ProcessEngine instance
	 * 
	 * @return ProcessEngine
	 */
	public static ProcessEngine getProcessEngine() {
		if (ProcessEngineProxy.processEngine == null) {
			// WARNING: Please check fe-core.properties inside core module
			if (FEPropertiesUtil.isTest()) {
				// Create a configuration
				Configuration configuration = new Configuration();
				
				// Build a process engine from a configuration
				ProcessEngineProxy.processEngine = configuration.buildProcessEngine();
			}
			else {
				try {
					ProcessEngineProxy.processEngine = 
						(ProcessEngine) getInitialContext( hostPort )
							.lookup(FEPropertiesUtil.getProcessEngineJndiName());
				} catch (Exception e) {
					logger.error("Could not find Process Engine.");
					throw new JbpmProcessException("Could not find Process Engine.", e);
				}
			}
		}
		return ProcessEngineProxy.processEngine;
	}
	
	/**
	 * Returns the repository service that provides access to the process repository. 
	 * 
	 * @return returns RepositoryService
	 */
	public static RepositoryService getRepositoryService() {
	      return ProcessEngineProxy.getProcessEngine().getRepositoryService();
	}
	
	/**
	 * Returns the execution service that provides access to the runtime executions repository. 
	 * 
	 * @return returns ExecutionService
	 */
	public static ExecutionService getExecutionService() {
	      return ProcessEngineProxy.getProcessEngine().getExecutionService();
	}
	
	/**
	 * Returns the task service that exposes the runtime human task lists. 
	 * 
	 * @return TaskService
	 */
	public static TaskService getTaskService() {
	      return ProcessEngineProxy.getProcessEngine().getTaskService();
	}
	
	/**
	 * Returns the history service that provides access to the history executions repository. 
	 * 
	 * @return returns HistoryService
	 */
	public static HistoryService getHistoryService() {
	      return ProcessEngineProxy.getProcessEngine().getHistoryService();
	}
	
	/**
	 * Returns the management service that exposes the management operations to operators that 
	 * have to keep the jBPM system up and running. 
	 * 
	 * @return returns ManagementService
	 */
	public static ManagementService getManagementService() {
	      return ProcessEngineProxy.getProcessEngine().getManagementService();
	}
	
	// -------------------------------------------------------------
	// UTILS
	// -------------------------------------------------------------
	
	@SuppressWarnings("unchecked")
	private static InitialContext getInitialContext(String hostPort)
			throws Exception {
		Hashtable props = getInitialContextProperties(hostPort);
		return new InitialContext(props);
	}

	@SuppressWarnings("unchecked")
	private static Hashtable getInitialContextProperties(String hostPort) {
		Hashtable props = new Hashtable();
		props.put("java.naming.factory.initial",
				"org.jnp.interfaces.NamingContextFactory");
		props.put("java.naming.factory.url.pkgs",
				"org.jboss.naming:org.jnp.interfaces");
		props.put("java.naming.provider.url", hostPort);

		return props;
	}

}
