/**
 * This file is part of FatturazioneElettronica - Signature and timestamp facilities
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Signature and timestamp facilities is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Signature and timestamp facilities is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Signature and timestamp facilities.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.signature;
import java.io.ByteArrayInputStream;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.cert.CertStore;
import java.security.cert.CertStoreException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.List;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMultipart;
import javax.mail.util.ByteArrayDataSource;

import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.tsp.TimeStampResponse;
import org.bouncycastle.util.encoders.Base64;

import com.sun.mail.util.BASE64DecoderStream;



public class VerificationUtils {
	/**
	 * Estrae la parte di firma digitale da un file m7m (Envelope generato da DiKe), in formato SMIME PKCS#7
	 * 
	 * @param file
	 * @return
	 */
	
	public static byte[] extractSignaturePart(byte[] file) {
		if (file == null)
			return null;
		final String contentDescriptionHeader = "Signed envelope";
		String s = new String(file);
		MimeMultipart multipart = null;
		try {
			multipart = new MimeMultipart(new ByteArrayDataSource(file,"multipart/mixed"));
			
			if (multipart.getCount() <= 0)
				return null;
			
			for (int i = 0; i<multipart.getCount(); i++){
				String[] header = multipart.getBodyPart(i).getHeader("Content-Description");
				for (int j = 0; j < header.length; j++) {
					if (contentDescriptionHeader.equals(header[j])){
						ByteArrayInputStream bis = (ByteArrayInputStream)multipart.getBodyPart(i).getContent();
						byte[] out = new byte [bis.available()];
						bis.read(out);
						return out;
					}
				}
			}
				
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		
		return null;
	}
	
	/**
	 * Estrae la parte di firma digitale da un file m7m (Envelope generato da DiKe), in formato SMIME PKCS#7
	 * 
	 * @param file
	 * @return
	 */
	public static byte[] extractSignedPayload(byte[] file) {
		if (file == null)
			return null;
		try {
			String s = new String (file);
			CMSSignedData signedData = new CMSSignedData(extractSignaturePart(file));
			return (byte[]) signedData.getSignedContent().getContent();
		} catch (CMSException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}	
	/**
	 * Estrae la parte di marca temporale da un file m7m (Envelope generato da DiKe), in formato SMIME PKCS#7
	 * Il file deve essere già decodificato Base64
	 * 
	 * @param file
	 * @return
	 */
	public static byte[] extractTimestampPart(byte[] file) {
		if (file == null)
			return null;
		final String contentDescriptionHeader = "time-stamp response";
		
		MimeMultipart multipart = null;
		try {
			multipart = new MimeMultipart(new ByteArrayDataSource(file,"multipart/mixed"));
			if (multipart.getCount() <= 0)
				return null;
			
			for (int i = 0; i<multipart.getCount(); i++){
				String[] header = multipart.getBodyPart(i).getHeader("Content-Description");
				for (int j = 0; j < header.length; j++) {
					if (contentDescriptionHeader.equals(header[j])){
						BASE64DecoderStream stream = (BASE64DecoderStream)multipart.getBodyPart(i).getContent();
						byte[] out = new byte [stream.available()];
						stream.read(out);
						return out;
					}
				}
			}
				
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		
		return null;
	}
	
	public static TimestampVerificationResult verifyTimeStamp(byte[] file) {
		byte[] firma = VerificationUtils.extractSignaturePart(file);
		byte[] marca = VerificationUtils.extractTimestampPart(file);

		TimestampVerificationResult result = VerificationUtils.verifyTimeStamp(
				firma, marca);
		
		return result;
	}
	/**
	 * 
	 * @param firmaPart : MIME part relativa alla firma
	 * @param marcaPart : MIME part relativa alla marca (già decodificata da Base64)
	 * @return
	 */
	protected static TimestampVerificationResult verifyTimeStamp(byte[] firmaPart, byte[] marcaPart) {
		TimestampVerificationResult out = new TimestampVerificationResult();
		try {
			
			TimeStampResponse resp = new TimeStampResponse(marcaPart);
			out.setTimestamp(resp.getTimeStampToken().getTimeStampInfo().getGenTime());

			// VERIFICARE PKIStatusInfo
			// TODO

			// VERIFICA TimeStampToken
			CertStore certStore = resp.getTimeStampToken()
					.getCertificatesAndCRLs("Collection", null);
			if (certStore != null) {
				Collection tsaCertificates = null;
				try {
					tsaCertificates = certStore.getCertificates(resp
							.getTimeStampToken().getSID());
				} catch (CertStoreException e) {
					e.printStackTrace();
					out.getErrorMessages().add(e.toString());
					out.setCertificateVerified(false);
				}

				if ((tsaCertificates == null) || (tsaCertificates.size() != 1)) {
					System.out.println("Exactly one certificate needed!");
					out.setCertificateVerified(false);
					out.getErrorMessages().add("Exactly one certificate needed!");
				}
				X509Certificate tsaCertificate = (X509Certificate) tsaCertificates
						.iterator().next();

				try {
					tsaCertificate.checkValidity();
					resp.getTimeStampToken().validate(tsaCertificate, null);
					out.setCertificateVerified(true);
				} catch (CertificateExpiredException e) {
					e.printStackTrace();
					out.setCertificateVerified(false);
					out.getErrorMessages().add(e.toString());
				} catch (CertificateNotYetValidException e) {
					e.printStackTrace();
					out.setCertificateVerified(false);
					out.getErrorMessages().add(e.toString());
				}

				//Timestamp imprint (e.g., digest embedded into the timestamp)
				byte[] digest = resp.getTimeStampToken().getTimeStampInfo()
						.getMessageImprintDigest();
				

				//Calculating digest from signature part
				MessageDigest md = null;
				if ("1.3.14.3.2.26".equals(resp.getTimeStampToken()
						.getTimeStampInfo().getMessageImprintAlgOID()))
					md = MessageDigest.getInstance("sha-1");
				else
					md = MessageDigest.getInstance("sha-256");

				md.update(firmaPart);
				byte[] originalDigest = md.digest();
				
				//Timestamp imprint and signature digest should be equal 
				if (ByteArrayUtils.equalsTo(originalDigest,digest ))
					out.setHashVerified(true);
				else
					out.setHashVerified(false);
				

			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return out;
	}
}
