/**
 * This file is part of FatturazioneElettronica - Web - Common
 *
 * Copyright © 2009-2014 EPOCA
 *
 * FatturazioneElettronica - Web - Common is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FatturazioneElettronica - Web - Common is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FatturazioneElettronica - Web - Common.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.fe.web.controller.generic;

import it.fe.core.par.user.ProfileDetail;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.hibernate.criterion.Criterion;

/**
 * Questa classe ridefinisce il metodo di accesso al datamodel della classe padre FilterableHandler, per verificare che l'utente 
 * correntemente loggato possa visualizzare il datamodel corretto.
 * 
 * Funge da base per la preparazione dei criteri di filtro basati sul ruolo/profilo utente e sullo stato delle fatture.
 * 
 * @author Stefano Monti
 *
 */
public abstract class ProfileFilterableHandler extends FilterableHandler {

	public static final String READ_WRITE = "rw";
	public static final String READ_ONLY = "r";
	public static final String NONE = "n";
	
	/**
	 * Variabile che indica se l'utente correntemente loggato ha un qualche profilo coerente con i permessi richiesti (RW o R)
	 * In caso positivo vale true, in caso negativo vale false --> posso usare questa variabile per determinare il rendering o meno dei datamodel
	 * associati alla portlet
	 */
	protected Boolean checkedPermission = null;
	
	
	public Set<ProfileDetail> validUserProfiles;
	protected Map<String, List<Criterion>> criteriProfilo = new HashMap<String, List<Criterion>>();
	
	
	public Boolean getCheckedPermission() {
		//TODO:remove
		if (checkedPermission == null)
			checkedPermission = determineUserPermissions();
		return checkedPermission;
	}

	public void setCheckedPermission(boolean permission) {
		this.checkedPermission = permission;
	}

	/**
	 * Per questioni di ottimizzazione, ridefinisco il metodo per fare update del modello, 
	 * tenendo conto dei permessi associati all'utente correntemente loggato.
	 */
	@Override
	public void updateDataModel() {
		//1. Gestione utenti: l'utente loggato ha uno stato compatibile con la visualizzazione degli elementi del datamodel?
		if (getCheckedPermission())
			super.updateDataModel();
		else 
			super.dataModel = null;
	}
	
	/**
	 * Per questioni di ottimizzazione, ridefinisco il metodo per fare update della dimensione del modello, 
	 * tenendo conto dei permessi associati all'utente correntemente loggato.
	 */
	@Override
	public void updateDataModelSize() {
		if (getCheckedPermission())
			super.updateDataModelSize();

		else 
			super.dataSize = 0;
	}
	
	
	
	
	/**
	 * Da ridefinire nelle sottoclassi con i criteri di gestione dei permessi sul datamodel
	 * Viene chiamato una volta sola al log dell'utente: per caricare eventuali modifiche al profilo, sarà necessario fare riloggare l'utente
	 * Metodo con side effect per questio di efficienza: oltre a restituire TRUE/FALSE se i profili utente sono coerenti 
	 * con le specifiche della portlet, (getRequiredPermission(), getRequiredState() ), riempie la struttura dati criteriProfilo 
	 * con i criteri di filtraggio che vengono dal profilo. 
	 *
	 * @return
	 */
	public abstract boolean determineUserPermissions() ;
	

}
